__author__ = 'weizhisong'

blast_result = open('/Users/weizhisong/Desktop/blast_result_273-4.txt')

def get_gene_list():
    gene_list = []
    for each_line in blast_result:
        query_name = each_line.split("\t")[0]
        if query_name in gene_list:
            pass
        else:
            gene_list.append(query_name)
    return gene_list

gene_list = get_gene_list()

for gene in gene_list:
    print gene
    for each_line in blast_result:
        query_name = each_line.split("\t")[0].strip('\n')
        subject_name = each_line.split("\t")[1].strip('\n')
        if query_name == gene:
            print "a"




 #   gene_cluster = []
  #  gene_cluster.append(gene)
   # b = gene_cluster
#    print gene_cluster
#    for each_line in blast_result:
#        print each_line

        #query_name = each_line.split("\t")[0].strip('\n')
        #subject_name = each_line.split("\t")[1].strip('\n')
        #print subject_name









#    subject_name = each_line.split("\t")[1]
#    e_value = each_line.split("\t")[10]
#    gene_list.append(query_name)
#    print query_name + "\t" + "\t" + subject_name






#    if len(gene_clusters) == 0:
#       gene_clusters.append(query_name)
#
#   else:
#      gene_clusters.append(subject_name)
# print gene_clusters






# file in
KEGG_DB_ko          = '/Users/songweizhi/Desktop/ko00001.keg'
annotation_table    = '/Users/songweizhi/Desktop/ko_relab_renamed.tsv'

# file out
annotation_table_new    = '/Users/songweizhi/Desktop/ko_relab_renamed_with_description.tsv'


Ds_description_dict = {}
for each_line in open(KEGG_DB_ko):
    if each_line[0] == 'D':
        each_line_split = each_line.strip().split(' ')
        current_D_id = each_line_split[6]
        current_D_description = ' '.join(each_line_split[7:])
        Ds_description_dict[current_D_id] = current_D_description

annotation_table_new_handle = open(annotation_table_new, 'w')
for each_line in open(annotation_table):
    if each_line.startswith('K'):
        ko_id = each_line.strip().split(':')[0]
        ko_desc = Ds_description_dict.get(ko_id, 'NA')
        annotation_table_new_handle.write('%s\t%s\n' % (each_line.strip(), ko_desc))
    else:
        annotation_table_new_handle.write('%s\t%s\n' % (each_line.strip(), 'NA'))
annotation_table_new_handle.close()



import argparse


def get_reads_id_in_sam(args):

    sam_file     = args['i']
    reads_id_txt = args['o']

    read_id_set = set()
    for each_line in open(sam_file):
        if not each_line.startswith('@'):
            each_line_split = each_line.strip().split('\t')
            read_id = each_line_split[0]
            read_id_set.add(read_id)

    reads_id_txt_handle = open(reads_id_txt, 'w')
    for each_read in read_id_set:
        reads_id_txt_handle.write(each_read + '\n')
    reads_id_txt_handle.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-i', required=True, help='input sam file')
    parser.add_argument('-o', required=True, help='output txt file')
    args = vars(parser.parse_args())
    get_reads_id_in_sam(args)



import os
import subprocess


def check_executables(program_list):

    not_detected_programs = []
    for needed_program in program_list:

        if subprocess.call(['which', needed_program], stdout=open(os.devnull, 'wb')) != 0:
            not_detected_programs.append(needed_program)

    if not_detected_programs != []:
        print('%s not detected, program exited!' % ','.join(not_detected_programs))
        exit()


check_executables(['mafft', 'hmmsearch'])



#!/usr/bin/env python3
import os
import glob
import shutil
import argparse


CheckM_output_parser_usage = '''
================================ CheckM_output_parser example commands ===============================

# only parse quality file
BioSAK CheckM_op_parser -i combined_qualities.txt -o bin_qualities.txt

# get the quality of qualified bins 
BioSAK CheckM_op_parser -i combined_qualities.txt -cpl 99 -o bin_qualities_cpl99.txt
BioSAK CheckM_op_parser -i combined_qualities.txt -cpl 99 -cont 5 -o bin_qualities_cpl99_cont5.txt

# get the quality of qualified bins and copy them into a separate folder
BioSAK CheckM_op_parser -i combined_qualities.txt -bin bin_folder -x fasta -cpl 99 -o bin_qualities_cpl99.txt
BioSAK CheckM_op_parser -i combined_qualities.txt -bin bin_folder -x fasta -cpl 99 -cont 5 -o bin_qualities_cpl99_cont5.txt

======================================================================================================
'''


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)
    os.mkdir(folder_to_create)


def CheckM_op_parser(args):

    combined_checkm_quality_file = args['i']
    input_bin_folder =             args['bin']
    bin_file_extension =           args['x']
    completeness_cutoff =          args['cpl']
    contamination_cutoff =         args['cont']
    output_file =                  args['o']

    qualitfied_bin_folder = None
    if input_bin_folder is not None:

        if input_bin_folder[-1] == '/':
            input_bin_folder = input_bin_folder[:-1]

        input_bin_folder_no_path = input_bin_folder
        if '/' in input_bin_folder:
            input_bin_folder_no_path = input_bin_folder.split('/')[-1]

        if (completeness_cutoff is not None) and (contamination_cutoff is None):
            qualitfied_bin_folder = '%s_complete%s' % (input_bin_folder_no_path, completeness_cutoff)

        if (completeness_cutoff is None) and (contamination_cutoff is not None):
            qualitfied_bin_folder = '%s_contain%s' % (input_bin_folder_no_path, contamination_cutoff)

        if (completeness_cutoff is not None) and (contamination_cutoff is not None):
            qualitfied_bin_folder = '%s_complete%s_contain%s' % (input_bin_folder_no_path, completeness_cutoff, contamination_cutoff)

    if (input_bin_folder is not None) and ((completeness_cutoff is not None) or (contamination_cutoff is not None)):
        force_create_folder(qualitfied_bin_folder)

    qualified_bin_num = 0
    output_file_handle = open(output_file, 'w')
    output_file_handle.write('Genome\tCompleteness\tContamination\n')
    for each_line in open(combined_checkm_quality_file):

        if (not each_line.startswith('-')) and (not each_line.startswith('  Bin')):
            quality_split = each_line.strip().split(' ')
            quality_split_new = []
            for each_line in quality_split:
                if each_line != '':
                    quality_split_new.append(each_line)

            genome_id = quality_split_new[0]
            completeness = float(quality_split_new[12])
            contamination = float(quality_split_new[13])

            if (completeness_cutoff is None) and (contamination_cutoff is None):
                output_file_handle.write('%s\t%s\t%s\n' % (quality_split_new[0], completeness, contamination))
                qualified_bin_num += 1

            if (completeness_cutoff is not None) and (contamination_cutoff is None):
                if completeness >= completeness_cutoff:
                    output_file_handle.write('%s\t%s\t%s\n' % (quality_split_new[0], completeness, contamination))
                    qualified_bin_num += 1
                    if input_bin_folder is not None:
                        os.system('cp %s/%s.%s %s/' % (input_bin_folder, genome_id, bin_file_extension, qualitfied_bin_folder))

            if (completeness_cutoff is None) and (contamination_cutoff is not None):
                if contamination <= contamination_cutoff:
                    output_file_handle.write('%s\t%s\t%s\n' % (quality_split_new[0], completeness, contamination))
                    qualified_bin_num += 1
                    if input_bin_folder is not None:
                        os.system('cp %s/%s.%s %s/' % (input_bin_folder, genome_id, bin_file_extension, qualitfied_bin_folder))

            if (completeness_cutoff is not None) and (contamination_cutoff is not None):
                if (completeness >= completeness_cutoff) and (contamination <= contamination_cutoff):
                    output_file_handle.write('%s\t%s\t%s\n' % (quality_split_new[0], completeness, contamination))
                    qualified_bin_num += 1
                    if input_bin_folder is not None:
                        os.system('cp %s/%s.%s %s/' % (input_bin_folder, genome_id, bin_file_extension, qualitfied_bin_folder))
    output_file_handle.close()

    # report
    if (completeness_cutoff is not None) or (contamination_cutoff is not None):
        print('The number of qualified genomes: %s' % qualified_bin_num)



#!/usr/bin/env python3

import os
import argparse
from BioSAK.global_functions import sep_path_basename_ext


fa2tree_usage = '''
============ fa2tree example commands ============

module load mafft/7.407
module load fasttree/2.1.11
BioSAK fa2tree -seq 16S.fa -t 12

==================================================
'''


def fa2tree(args):

    seq_file    = args['seq']
    thread_num  = args['t']

    seq_file_path, seq_file_basename, seq_file_ext = sep_path_basename_ext(seq_file)

    aln_file  = '%s/%s.aln'     % (seq_file_path, seq_file_basename)
    tree_file = '%s/%s.newick'  % (seq_file_path, seq_file_basename)

    cmd_mafft    = 'mafft --quiet --maxiterate 1000 --adjustdirection --thread %s --globalpair %s > %s' % (thread_num, seq_file, aln_file)
    print(cmd_mafft)
    os.system(cmd_mafft)

    cmd_fasttree = 'fasttree -nt -quiet %s > %s' % (aln_file, tree_file)
    print(cmd_fasttree)
    os.system(cmd_fasttree)

    print('Tree exported to %s' % tree_file)


if __name__ == '__main__':

    fa2tree_parser = argparse.ArgumentParser(usage=fa2tree_usage)
    fa2tree_parser.add_argument('-seq',  required=True,                        help='sequence file')
    fa2tree_parser.add_argument('-t',    required=False, type=int, default=1,  help='number of threads, default: 1')
    args = vars(fa2tree_parser.parse_args())
    fa2tree(args)



import argparse

parser = argparse.ArgumentParser()

parser.add_argument('-sam',
                    required=True,
                    help='Input sam file')

parser.add_argument('-ctgs',
                    required=True,
                    help='Contig list')

parser.add_argument('-option',
                    required=True,
                    type=int,
                    help="'0' to export unmapped reads, '1' to export mapped reads")

parser.add_argument('-out',
                    required=True,
                    help='Output fasta file')

args = vars(parser.parse_args())
sam_file = args['sam']
output_file = args['out']
ctgs_file = args['ctgs']
option = args['option']


# get contig list
ctg_list = []
for each_ctg in open(ctgs_file):
    each_ctg = each_ctg.strip()
    ctg_list.append(each_ctg)


# export reads
output = open(output_file, 'w')
for each in open(sam_file):
    if each.startswith('@'):
        pass
    else:
        each_split = each.strip().split('\t')
        query_name = each_split[0]
        ref_name = each_split[2]
        query_seq = each_split[9]

        if option == 1:
            if ref_name in ctg_list:
                output.write('>%s\n' % query_name)
                output.write('%s\n' % query_seq)

        if option == 0:
            if ref_name not in ctg_list:
                output.write('>%s\n' % query_name)
                output.write('%s\n' % query_seq)

output.close()




# file in
pwd_cog_20_def_tab      = '/Users/songweizhi/Desktop/cog-20.def.tab'
annotation_table        = '/Users/songweizhi/Desktop/eggnog_relab.tsv'

# file out
annotation_table_new    = '/Users/songweizhi/Desktop/eggnog_relab_with_description.tsv'


# get cog_id_to_description_dict (cognames2003-2014.tab)
cog_id_to_description_dict = {}
for cog_id_to_cate_des in open(pwd_cog_20_def_tab, encoding='windows-1252'):
    if not cog_id_to_cate_des.startswith('#'):
        cog_id_to_cate_des_split = cog_id_to_cate_des.strip().split('\t')
        cog_id = cog_id_to_cate_des_split[0]
        cog_cate = cog_id_to_cate_des_split[1]
        cog_des = cog_id_to_cate_des_split[2]
        cog_id_to_description_dict[cog_id] = cog_des


annotation_table_new_handle = open(annotation_table_new, 'w')
for each_line in open(annotation_table):
    if each_line.startswith('COG'):
        cog_id = each_line.strip().split('\t')[0]
        if '|' in cog_id:
            cog_id = cog_id.split('|')[0]
        cog_desc = cog_id_to_description_dict.get(cog_id, 'NA')
        annotation_table_new_handle.write('%s\t%s\n' % (each_line.strip(), cog_desc))
    else:
        annotation_table_new_handle.write('%s\t%s\n' % (each_line.strip(), 'NA'))
annotation_table_new_handle.close()




from ete3 import NCBITaxa, Tree, TreeStyle, NodeStyle, TextFace


def plot_tree(tree_newick, tree_title):

    tree = Tree(tree_newick, format = 1)
    # set tree parameters
    ts = TreeStyle()
    ts.mode = "r"  # tree model: 'r' for rectangular, 'c' for circular
    ts.show_leaf_name = 0
    # set tree title text parameters
    ts.title.add_face(TextFace(tree_title,
                               fsize = 8,
                               fgcolor = 'black',
                               ftype = 'Arial',
                               tight_text = False),
                      column = 0)  # tree title text setting
    # set layout parameters
    ts.rotation = 0  # from 0 to 360
    ts.show_scale = False
    ts.margin_top = 10  # top tree image margin
    ts.margin_bottom = 10  # bottom tree image margin
    ts.margin_left = 10  # left tree image margin
    ts.margin_right = 10  # right tree image margin
    ts.show_border = False  # set tree image border
    ts.branch_vertical_margin = 3  # 3 pixels between adjancent branches

    # set tree node style
    for each_node in tree.traverse():
        # leaf node parameters
        if each_node.is_leaf():
            ns = NodeStyle()
            ns["shape"] = "circle"  # dot shape: circle, square or sphere
            ns["size"] = 0  # dot size
            ns['hz_line_width'] = 0.5  # branch line width
            ns['vt_line_width'] = 0.5  # branch line width
            ns['hz_line_type'] = 0  # branch line type: 0 for solid, 1 for dashed, 2 for dotted
            ns['vt_line_type'] = 0  # branch line type
            ns["fgcolor"] = "blue"  # the dot setting
            each_node.add_face(TextFace(each_node.name,
                                        fsize = 5,
                                        fgcolor = 'black',
                                        tight_text = False,
                                        bold = False),
                               column = 0,
                               position = 'branch-right')  # leaf node the node name text setting

            each_node.set_style(ns)

        # non-leaf node parameters
        else:
            nlns = NodeStyle()
            nlns["size"] = 0  # dot size
            #nlns["rotation"] = 45
            each_node.add_face(TextFace(each_node.name[:12],

                                        fsize = 4,
                                        fgcolor = 'black',
                                        tight_text = False,
                                        bold = False),
                               column = 5,
                               position = 'branch-top')  # non-leaf node name text setting)

            each_node.set_style(nlns)

    tree.render('/Users/songweizhi/Desktop/Tree_' + tree_title + '.png', w = 900, units = "px", tree_style = ts)  # set figures size


taxon_id_assignment_file = '/Users/songweizhi/Desktop/taxon_assignment_168.txt'
taxdump_file = '/Users/songweizhi/Desktop/taxdump_customized.tar.gz'

# get taxon_id_list
taxon_ids = open(taxon_id_assignment_file)
taxon_id_list = []
for each in taxon_ids:
    each_split = each.strip().split('\t')
    taxon_id = each_split[1]
    taxon_id_list.append(taxon_id)

# print(taxon_id_list)
# print(len(taxon_id_list))

ncbi = NCBITaxa()
#ncbi.update_taxonomy_database(taxdump_file)

tree_phylo = ncbi.get_topology(taxon_id_list, intermediate_nodes=1)

tree_phylo_newick = tree_phylo.write(format=8)

for each_node in tree_phylo.traverse():
    node_name_list = []
    node_name_list.append(each_node.name)
    if node_name_list == ['']:
        pass
    else:
        if each_node.is_leaf():
            # change bin id to bin name
            each_node.name = ncbi.get_taxid_translator(node_name_list)[int(each_node.name)]
            # add group information to bin name

        # for non-leaf node name, only display the first 12 characters
        else:
            if len(ncbi.get_taxid_translator(node_name_list)[int(each_node.name)]) <= 12:
                each_node.name = ncbi.get_taxid_translator(node_name_list)[int(each_node.name)]
            else:
                each_node.name = ncbi.get_taxid_translator(node_name_list)[int(each_node.name)][0:12] + '.'

print(tree_phylo)
tree_phylo_newick_2 = tree_phylo.write(format=8)

print(tree_phylo_newick_2)

plot_tree(tree_phylo_newick, '168 bins_id')
plot_tree(tree_phylo_newick_2, '168 bins_name')











import os
from datetime import datetime
from Bio import AlignIO
from Bio.Phylo.TreeConstruction import DistanceCalculator
from Bio.Phylo.TreeConstruction import DistanceTreeConstructor
from ete3 import Tree, ClusterTree
import numpy as np
from scipy.cluster.hierarchy import linkage
from scipy.cluster.hierarchy import fcluster




wd = '/Users/songweizhi/Desktop/get_distance_distribution'
tree_plotter_R = '~/R_scripts/newick_tree/tree_plotter.R'
os.chdir(wd)

aln_file = 'species_tree2.aln'
tree_file_out = 'species_tree2.newick'
taxon_assignment = 'taxon_assignment.txt'


# read in bin id to taxon dict
bin_id_to_taxon_dict = {}
for each in open(taxon_assignment):
    each_split = each.strip().split(',')
    bin_id_to_taxon_dict[each_split[0]] = each_split[1]


# read in tree file
aln_handle = AlignIO.read(aln_file, "fasta")

# get distance matrix
print(datetime.now().strftime('%Y-%m-%d %H:%M:%S') + ' Get distance matrix')
calculator = DistanceCalculator('identity')
# calculator = DistanceCalculator('blosum62')
distance_matrix = calculator.get_distance(aln_handle)
#print(distance_matrix)

# construct a tree
# see the difference between UPGMA, NJ and Maximum likelihood
print(datetime.now().strftime('%Y-%m-%d %H:%M:%S') + ' Construct tree')
constructor = DistanceTreeConstructor()
tree_NJ = constructor.nj(distance_matrix)

# turn to newick format and write out
print(datetime.now().strftime('%Y-%m-%d %H:%M:%S') + ' Turn to newick format and write out')
tree_NJ_newick = Tree(tree_NJ.format('newick'),format=3)
for each_leaf in tree_NJ_newick:
    #print(each_leaf.name)
    each_leaf.name = bin_id_to_taxon_dict[each_leaf.name]


tree_NJ_newick.write(format=3, outfile=tree_file_out)

# plot the tree
os.system('Rscript %s -t %s' % (tree_plotter_R, tree_file_out))


# get full matrix
seq_name_list = distance_matrix.names

all_distances_lol = []
for each_seq_1 in seq_name_list:
    current_seq_list = []
    for each_seq_2 in seq_name_list:
        distance_between_seqs = distance_matrix[each_seq_1, each_seq_2]
        distance_between_seqs = float("{0:.1f}".format(distance_between_seqs))
        current_seq_list.append(distance_between_seqs)
    all_distances_lol.append(current_seq_list)

all_distances_lol_array = np.array(all_distances_lol)

print(all_distances_lol_array)


cluster = linkage(all_distances_lol_array, method='weighted')

flat_clusters = fcluster(cluster, 0.75, criterion='distance')

print(flat_clusters)

print(seq_name_list)




out = open('clustering.txt', 'w')
n = 0
for each in seq_name_list:
    #print('%s\t%s\t%s' % (flat_clusters[n], each, bin_id_to_taxon_dict[each]))
    out.write('%s\t%s\t%s\n' % (bin_id_to_taxon_dict[each], flat_clusters[n], each))
    n += 1

out.close()


os.system('cat clustering.txt | sort > clustering_sorted.txt')
# distance_between_seqs = distance_matrix[seq_id_1, seq_id_2]

for each in open('clustering_sorted.txt'):
    print(each.strip())

print('The number of sequences: %s' % len(seq_name_list))



import os
import argparse
from Bio import SeqIO

fa2id_usage = '''
============== fa2id example command ==============

BioSAK fa2id -i gnm.fa -o ctg_id.txt
BioSAK fa2id -i reads.fastq -o reads_id.txt -fq

===================================================
'''


def fa2id(args):

    fasta_in   = args['i']
    output_txt = args['o']
    in_fastq   = args['fq']

    seq_in_format = 'fasta'
    if in_fastq is True:
        seq_in_format = 'fastq'

    output_txt_handle = open(output_txt, 'w')
    for each_seq in SeqIO.parse(fasta_in, seq_in_format):
        output_txt_handle.write(each_seq.id + '\n')
    output_txt_handle.close()


if __name__ == '__main__':

    fa2id_usage_parser = argparse.ArgumentParser(usage=fa2id_usage)
    fa2id_usage_parser.add_argument('-i',  required=True, help='input fasta/fastq file')
    fa2id_usage_parser.add_argument('-o',  required=True, help='output txt')
    fa2id_usage_parser.add_argument('-fq', required=False, action="store_true",  help='in fastq format, default: fasta')
    args = vars(fa2id_usage_parser.parse_args())
    fa2id(args)



import os
from Bio import SeqIO


# input files
input_gbk_file = '/Users/songweizhi/Desktop/MT_contigs_lt500bp_prodigal_gbk/MT9_contig_lt500.gbk'
ctg_depth_file = '/Users/songweizhi/Desktop/MT_contigs_lt500bp_prodigal_gbk/MT9_contig_all_depth.txt'


# prepare file name
gbk_file_path, gbk_file_name = os.path.split(input_gbk_file)
gbk_file_name_no_extension, gbk_file_extension = os.path.splitext(gbk_file_name)
output_file = '%s/%s_gene_depth.txt' % (gbk_file_path, gbk_file_name_no_extension)


# get ctg_depth_dict
ctg_depth_dict = {}
for ctg in open(ctg_depth_file):
    if not ctg.startswith('contigName'):
        ctg_split = ctg.strip().split('\t')
        ctg_id = ctg_split[0]
        ctg_depth = ctg_split[2]
        ctg_depth_dict[ctg_id] = ctg_depth


output_file_handle = open(output_file, 'w')
output_file_handle.write('Gene\tContig\tCtg_length\tCtg_depth\n')
for seq_record in SeqIO.parse(input_gbk_file, 'genbank'):

    seq_id = seq_record.id
    seq_len = len(seq_record.seq)
    seq_depth = 'NA'

    if seq_id in ctg_depth_dict:
        seq_depth = ctg_depth_dict[seq_id]

    for feature in seq_record.features:
        feature_id = feature.qualifiers['locus_tag'][0]
        output_file_handle.write('%s\t%s\t%s\t%s\n' % (feature_id, seq_id, seq_len, seq_depth))

output_file_handle.close()











import os
import glob
import argparse
from Bio import SeqIO


get_MAG_reads_long_usage = '''
================================== get_MAG_reads_long example commands ==================================

BioSAK get_MAG_reads_long -r LongReads.fq -s assembly.sam -m MAG_1.fa -o MAG_1_reads -l 3000
BioSAK get_MAG_reads_long -r LongReads.fq -s assembly.sam -m MAG_dir -x fa -o extracted_reads -l 0
BioSAK get_MAG_reads_long -r LongReads.fq -s assembly.sam -m MAG_dir -x fa -o extracted_reads -l 10000

# Note
The input sam file is obtained by mapping filtered reads to all flye produced contigs
minimap2 -d assembly.mmi assembly.fasta
minimap2 -t 12 --secondary=no -a assembly.mmi long_reads.fastq > assembly.no.secondary.sam 

=========================================================================================================
'''


def get_MAG_reads_long(args):

    reads_file          = args['r']
    sam_file            = args['s']
    mag_folder          = args['m']
    mag_ext             = args['x']
    min_read_len        = args['l']
    extracted_read_dir  = args['o']

    # create folder
    if os.path.isdir(extracted_read_dir) is True:
        print('Output folder detected, program exited!')
        print(extracted_read_dir)
        exit()

    mag_list = []
    if os.path.isfile(mag_folder) is True:
        mag_list = [mag_folder]
    elif os.path.isdir(mag_folder) is True:
        mag_re = '%s/*.%s' % (mag_folder, mag_ext)
        mag_list = glob.glob(mag_re)
        if len(mag_list) == 0:
            print('No MAG detected, program exited!')
            exit()
    else:
        print('No MAG file/dir detected, program exited!')
        exit()

    print('getting ctg_to_gnm dict')
    ctg_to_gnm_dict = {}
    ctgs_in_multi_mags_set = set()
    for each_gnm in mag_list:
        gnm_id = '.'.join(os.path.basename(each_gnm).strip().split('.')[:-1])
        for each_seq in SeqIO.parse(each_gnm, 'fasta'):
            ctg_id = each_seq.id
            if ctg_id not in ctg_to_gnm_dict:
                ctg_to_gnm_dict[ctg_id] = gnm_id
            else:
                ctgs_in_multi_mags_set.add(ctg_id)

    # check if there is contig in multiple MAGs
    if len(ctgs_in_multi_mags_set) > 0:
        print('The following contigs found in multiple MAGs, proceed anyway!')
        print(','.join(sorted([i for i in ctgs_in_multi_mags_set])))

    print('getting read_to_gnm_dict')
    read_to_gnm_dict = {}
    for each_line in open(sam_file):
        if not each_line.startswith('@'):
            each_line_split = each_line.strip().split('\t')
            read_id = each_line_split[0]
            ref_id = each_line_split[2]
            ref_gnm = ctg_to_gnm_dict.get(ref_id, 'NA')
            if ref_gnm != 'NA':
                if read_id not in read_to_gnm_dict:
                    read_to_gnm_dict[read_id] = {ref_gnm}
                else:
                    read_to_gnm_dict[read_id].add(ref_gnm)

    os.system('mkdir %s' % extracted_read_dir)

    print('Extracting reads')
    for each_read in SeqIO.parse(reads_file, 'fastq'):
        read_id = each_read.id
        read_len = len(each_read.seq)
        read_gnms = read_to_gnm_dict.get(read_id, {})
        if read_len >= min_read_len:
            if read_gnms != {}:
                for each_gnm in read_gnms:
                    pwd_read_file = '%s/%s_reads_min%sbp.fastq' % (extracted_read_dir, each_gnm, min_read_len)
                    pwd_read_file_handle = open(pwd_read_file, 'a')
                    SeqIO.write(each_read, pwd_read_file_handle, 'fastq')
                    pwd_read_file_handle.close()

    # print flye cmds
    print('Suggested flye commands:')
    for each_gnm in sorted(mag_list):
        gnm_id = '.'.join(os.path.basename(each_gnm).strip().split('.')[:-1])
        flye_cmd = 'flye --meta --threads 12 --out-dir flye_reassemble_wd_%s --nano-raw %s_reads_min%sbp.fastq' % (gnm_id, gnm_id, min_read_len)
        print(flye_cmd)

    print('Done!')


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()
    parser.add_argument('-r',    required=True,                             help='reads file')
    parser.add_argument('-s',    required=True,                             help='sam file')
    parser.add_argument('-m',    required=True,                             help='MAG file/folder')
    parser.add_argument('-x',    required=False, default='fasta',           help='MAG extension, default: fasta')
    parser.add_argument('-l',    required=False, type=int, default=5000,    help='minimal read length (in bp), default: 5000')
    parser.add_argument('-o',    required=True,                             help='Output folder')
    args = vars(parser.parse_args())
    get_MAG_reads_long(args)



import pandas as pd
from copy import deepcopy


def drop_small_value_rows(df_in, min_value):

    df_in_copy = deepcopy(df_in)
    df_in_copy['max_in_row'] = df_in_copy.max(axis=1)
    df_out_tmp = df_in_copy[df_in_copy['max_in_row'] >= min_value]
    df_out = df_out_tmp.drop(labels='max_in_row', axis=1)

    return df_out


def drop_small_value_cols_and_rows(input_file, min_value, output_file):
    input_df = pd.read_csv(open(input_file), delimiter='\t', header=0, index_col=0)

    input_df_filtered_row = drop_small_value_rows(input_df, min_value)
    input_df_filtered_row_t = input_df_filtered_row.transpose()
    input_df_filtered_row_col_t = drop_small_value_rows(input_df_filtered_row_t, min_value)
    input_df_filtered_row_col = input_df_filtered_row_col_t.transpose()

    input_df_filtered_row_col.to_csv(output_file, sep='\t')


input_file = '/Users/songweizhi/Desktop/SpongeEMP_circos.txt'
min_value = 5
output_file = '/Users/songweizhi/Desktop/SpongeEMP_circos_filtered.txt'


drop_small_value_cols_and_rows(input_file, min_value, output_file)




import requests
from bs4 import BeautifulSoup


# https://www.dataquest.io/blog/web-scraping-tutorial-python/






page = requests.get("http://dataquestio.github.io/web-scraping-pages/simple.html")
print(page)
soup = BeautifulSoup(page.content, 'html.parser')

html = list(soup.children)[2]


soup = BeautifulSoup(page.content, 'html.parser')
soup.find_all('p')


"""
Distribution plot options
# https://stanford.edu/~mwaskom/software/seaborn/examples/distplot_options.html
=========================

"""
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt

sns.set(style="white", palette="muted", color_codes=True)
rs = np.random.RandomState(10)

# Set up the matplotlib figure
f, axes = plt.subplots(2, 2, figsize=(8, 8), sharex=False)
sns.despine(left=False)

# Generate a random univariate dataset
d = rs.normal(size=100)

# Plot a simple histogram with binsize determined automatically
sns.distplot(d, kde=False, color="b", ax=axes[0, 0])

# Plot a kernel density estimate and rug plot
sns.distplot(d, hist=False, rug=True, color="r", ax=axes[0, 1])

# Plot a filled kernel density estimate
sns.distplot(d, hist=False, color="g", kde_kws={"shade": True}, ax=axes[1, 0])

# Plot a historgram and kernel density estimate
sns.distplot(d, color="m", ax=axes[1, 1])

plt.setp(axes, yticks=[])
#plt.tight_layout()
plt.show()


#!/usr/bin/env python

import os
import argparse
import itertools
from ete3 import Tree
from string import ascii_uppercase


usage = """

python /srv/scratch/z5039045/Scripts/cluster_to_grouping_file.py -t species_tree.newick -c grouping_out.txt -g grouping.txt

"""

def get_group_index_list():
    def iter_all_strings():
        size = 1
        while True:
            for s in itertools.product(ascii_uppercase, repeat=size):
                yield "".join(s)
            size += 1

    group_index_list = []
    for s in iter_all_strings():
        group_index_list.append(s)
        if s == 'ZZ':
            break
    return group_index_list


def cluster_2_grouping_file(cluster_file, grouping_file):

    t1 = 'cluster_tmp1.txt'
    t1_sorted = 'cluster_tmp1_sorted.txt'

    t1_handle = open(t1, 'w')
    for each in open(cluster_file):
        if not each.startswith(','):
            genome_id = each.strip().split(',')[0]
            cluster_id = each.strip().split(',')[1]
            t1_handle.write('%s,%s\n' % (cluster_id, genome_id))
    t1_handle.close()

    os.system('cat %s | sort > %s' % (t1, t1_sorted))
    group_index_list = get_group_index_list()
    grouping_file_handle = open(grouping_file, 'w')

    current_cluster_name = ''
    group_index_no = 0
    for each in open(t1_sorted):
        cluster_name = each.strip().split(',')[0]
        genome_name = each.strip().split(',')[1]

        if current_cluster_name == '':
            current_cluster_name = cluster_name
            grouping_file_handle.write('%s,%s\n' % (group_index_list[group_index_no], genome_name))
        elif current_cluster_name == cluster_name:
            grouping_file_handle.write('%s,%s\n' % (group_index_list[group_index_no], genome_name))
        elif current_cluster_name != cluster_name:
            current_cluster_name = cluster_name
            group_index_no += 1
            grouping_file_handle.write('%s,%s\n' % (group_index_list[group_index_no], genome_name))

    os.remove(t1)
    os.remove(t1_sorted)


def get_id2name_dict(grouping_file):
    id2name_dict = {}
    for each in open(grouping_file):
        each_split = each.strip().split(',')
        group_id = each_split[0]
        genome_name = each_split[1]
        id2name_dict[genome_name] = group_id
    return id2name_dict


def add_grouping_to_tree(tree_file, id2name_dict):
    # rename leaf node name
    tree_file_name, tree_file_ext = os.path.splitext(tree_file)
    tree_file_with_grouping = '%s_with_grouping%s' % (tree_file_name, tree_file_ext)
    gene_tree = Tree(tree_file, format=3)
    gene_tree.resolve_polytomy(recursive=True)  # solving multifurcations

    # change gene tree leaf name for Ranger-DTL
    # for leaf_node in gene_tree:
    #     leaf_node_name_new = '%s_%s' % (id2name_dict[leaf_node.name], leaf_node.name)
    #     leaf_node.name = leaf_node_name_new

    for leaf_node in gene_tree:
        leaf_node_name_new = '%s' % (id2name_dict[leaf_node.name])
        leaf_node.name = leaf_node_name_new


    # write tree to new file
    tree_file_with_grouping_handle = open(tree_file_with_grouping, 'w')
    tree_file_with_grouping_handle.write('%s\n' % (gene_tree.write(format=3)))
    tree_file_with_grouping_handle.close()


parser = argparse.ArgumentParser()

parser.add_argument('-c',
                    required=True,
                    help='cluster file')

parser.add_argument('-t',
                    required=True,
                    help='tree file')

parser.add_argument('-g',
                    required=True,
                    help='grouping file')

args = vars(parser.parse_args())

tree_file = args['t']
cluster_file = args['c']
grouping_file = args['g']


cluster_2_grouping_file(cluster_file, grouping_file)
id2name_dict = get_id2name_dict(grouping_file)
add_grouping_to_tree(tree_file, id2name_dict)



import requests
import json
from lxml import etree
import time
import datetime


#获取网页相应内容
def get_one_page(url):

    headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/65.0.3325.181 Safari/537.36'}
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        return response.text
    else:
        return None



n = 1
out_html = '/Users/songweizhi/Desktop/out_html.txt'

# 查找关键字网址
url = 'https://microbiomejournal.biomedcentral.com/articles?searchType=journalSearch&sort=PubDate&page={num}'.format(num=n)
print(url)

# 获取相应内容
headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/65.0.3325.181 Safari/537.36'}
response = requests.get(url, headers=headers)
html = response.text
print(html)
out_html_handle = open(out_html, 'w')
out_html_handle.write(html)
out_html_handle.close()

# xpath解析文献href
html = etree.HTML(html)
href = html.xpath('//h3/a[@data-test="title-link"]/@href')
print(href)




#
# # xpath解析文献href
# html = etree.HTML(html)
# href = html.xpath('//h3/a[@data-test="title-link"]/@href')
#
# # 对每个href进行关键字筛选
# for i in href:
#         # 单篇文献url
#         url = 'https://microbiomejournal.biomedcentral.com' + str(i)
#         doi = i.replace('/articles', '')
#         url_pdf = 'https://microbiomejournal.biomedcentral.com/track/pdf' + doi + '#page=1'
#         print(url_pdf)
#         html = get_one_page(url)
#         name = []
#         # 判断关键字是否存在于文献中
#         for item in parse_one_page(html):
#                 with open('data.csv', 'a', encoding='utf-8') as f:
#                         f.write(json.dumps(item['Title'], ensure_ascii=False) + ',')
#                         f.write(json.dumps(item['DOI'], ensure_ascii=False) + ',')
#                         f.write(json.dumps(item['Accesses'], ensure_ascii=False) + ',')
#                         f.write(json.dumps(item['Citations'], ensure_ascii=False) + ',')
#                         f.write(json.dumps(item['Altmetric'], ensure_ascii=False) + ',')
#                         f.write(json.dumps(item['published'], ensure_ascii=False) + ',')
#                         f.write(json.dumps(item['author'], ensure_ascii=False) + '\n')
#                         time.sleep(2)
#                         f.close()
#                 print('======读取pdf中=====')
#                 r = get_one_page_pdf(url_pdf)
#                 print('======保存pdf中=====')
#                 with open('data_pdf/' + item['Title'][0] + '.pdf', 'wb') as p:
#                         p.write(r.content)
#                         print('======保存完毕=====')
#                         p.close()
#                 time.sleep(2)
#




import os
import argparse
import multiprocessing as mp

exe_cmds_usage = '''
====== exe_cmds example command ======

BioSAK exe_cmds -c cmds.txt -t 12

# one command per line
======================================
'''


def exe_cmds(args):

    cmd_file    = args['c']
    num_threads = args['t']

    cmd_set = set()
    for each_cmd in open(cmd_file):
        cmd_set.add(each_cmd.strip())

    print('Running %s commands with %s cores' % (len(cmd_set), num_threads))
    pool = mp.Pool(processes=num_threads)
    pool.map(os.system, sorted([i for i in cmd_set]))
    pool.close()
    pool.join()
    print('Done!')


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-c',     required=True,                       help='cmds file')
    parser.add_argument('-t',     required=False, type=int, default=1, help='number of threads')
    args = vars(parser.parse_args())
    exe_cmds(args)



import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


def boxplot_with_dots(num_lol, name_list, output_plot):

    data = [np.array(i) for i in num_lol]

    box_plot = plt.boxplot(data, labels=name_list, patch_artist=True,
                           whiskerprops=dict(color='lightblue', linewidth=2), capprops=dict(color='lightblue'))

    # set the color pf box
    for box in box_plot['boxes']:
        box.set(linewidth=0)
        box.set_facecolor('lightblue')

    # add dots, https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.plot.html
    col_index = 1
    for num_arrary in data:
        plt.plot(np.random.normal(col_index, 0.02, len(num_arrary)), num_arrary, '.', alpha=0.8, color='orange',
                 markersize=6, markeredgewidth=0)
        col_index += 1

    # write out
    plt.tight_layout()
    plt.savefig(output_plot, bbox_inches='tight', dpi=300)
    plt.close()


num_lol     = [[1, 2, 3, 4], [4, 5, 6, 7], [6, 7, 8, 9]]
name_list   = ['apple', 'banana', 'orange']
output_plot = '/Users/songweizhi/Desktop/boxplot.png'

boxplot_with_dots(num_lol, name_list, output_plot)




output_df_1                                 = '/Users/songweizhi/Desktop/1.txt'
output_df_2                                 = '/Users/songweizhi/Desktop/2.txt'

intersect_file_list                         = {'mag_1', 'mag_4', 'mag_5', 'APA_bin_1', 'mag_2', 'mag_3'}
pwys_to_detect                              = ['|3HP|', '|4HB3HP|', '|CALVIN-PWY|', '|CODHa-PWY|', '|CODHb-PWY|', '|DC4HB|', '|rTCA1|', '|rTCA2|']
pwy_to_key_enzyme_dict                      = {'|rTCA1|': [['citas', 'citbs']], '|rTCA2|': [['CCSs', 'CCSL', 'CCL']], '|4HB3HP|': [['HBD_alignments']], '|3HP|': [['TIGR04253']], '|DC4HB|': [['HBD_alignments']], '|CALVIN-PWY|': [['PRK'], ['PRK2']], '|CODHb-PWY|': [['MeTr_alignments']], '|CODHa-PWY|': [['MeTr_alignments']]}
pwy_completeness_cutoff                     = 80

detected_key_enzymes_dict                   = {'mag_1': {'TIGR04253'},
                                               'mag_4': {'PRK', 'TIGR04253', 'PRK2'},
                                               'mag_5': {'citas', 'citbs'},
                                               'APA_bin_1': set(),
                                               'mag_2': {'HBD_alignments'},
                                               'mag_3': {'PRK'}}

pwy_completeness_dict                       = {'mag_1': {'|3HP|': 80.0},
                                               'mag_4': {'|3HP|': 80.0, '|CALVIN-PWY|': 84.0},
                                               'mag_5': {'|rTCA1|': 80.0}, 'APA_bin_1': {},
                                               'mag_2': {'|4HB3HP|': 31.0, '|DC4HB|': 53.0},
                                               'mag_3': {'|CALVIN-PWY|': 92.0}}

mag_to_paths_with_qualified_key_enzyme_pct  = {'mag_1': ['|3HP|'],
                                               'mag_4': ['|3HP|', '|CALVIN-PWY|'],
                                               'mag_5': ['|rTCA1|'],
                                               'mag_2': ['|4HB3HP|', '|DC4HB|'],
                                               'mag_3': ['|CALVIN-PWY|']}

# write out as data matrix
output_df_1_handle = open(output_df_1, 'w')
output_df_2_handle = open(output_df_2, 'w')
header_printed = False
for genome in sorted(intersect_file_list):
    current_genome_header_list = []
    current_genome_value_list = []
    current_genome_header_list_2 = []
    current_genome_value_list_2 = []
    for pathway in pwys_to_detect:
        current_pathway_key_enzymes = pwy_to_key_enzyme_dict[pathway]
        group_header_top = []
        group_value_top = []
        for key_enzyme_g in current_pathway_key_enzymes:
            group_header = '_n_'.join(key_enzyme_g)
            group_value = []
            for key_enzyme in key_enzyme_g:
                if key_enzyme in detected_key_enzymes_dict[genome]:
                    group_value.append('1')
                else:
                    group_value.append('0')

            group_value_top.append('_n_'.join(group_value))
            group_header_top.append(group_header)

        current_genome_header_list.append('%s__%s' % (pathway, '_v_'.join(group_header_top)))
        current_genome_value_list.append('_v_'.join(group_value_top))

        current_genome_header_list.append('%s_completeness' % pathway)
        current_genome_header_list.append('%s_found' % pathway)
        current_genome_header_list_2.append(pathway[1:-1])

        if pathway in pwy_completeness_dict[genome]:
            current_genome_value_list.append(str(pwy_completeness_dict[genome][pathway]))
        else:
            current_genome_value_list.append('na')

        # if (pathway in mag_to_paths_with_qualified_key_enzyme_pct[genome]) and (pwy_completeness_dict[genome][pathway] >= pwy_completeness_cutoff):
        #     current_genome_value_list.append('1')
        #     current_genome_value_list_2.append('1')
        # else:
        #     current_genome_value_list.append('0')
        #     current_genome_value_list_2.append('0')

        value_to_append = '0'
        if genome in mag_to_paths_with_qualified_key_enzyme_pct:
            if pathway in mag_to_paths_with_qualified_key_enzyme_pct[genome]:
                if genome in pwy_completeness_dict:
                    if pathway in pwy_completeness_dict[genome]:
                        if pwy_completeness_dict[genome][pathway] >= pwy_completeness_cutoff:
                            value_to_append = '1'
        current_genome_value_list.append(value_to_append)
        current_genome_value_list_2.append(value_to_append)

    if header_printed is False:
        output_df_1_handle.write('Genome\t%s\n' % '\t'.join(current_genome_header_list))
        output_df_2_handle.write('Genome\t%s\n' % '\t'.join(current_genome_header_list_2))
        header_printed = True
    output_df_1_handle.write('%s\t%s\n' % (genome, '\t'.join(current_genome_value_list)))
    output_df_2_handle.write('%s\t%s\n' % (genome, '\t'.join(current_genome_value_list_2)))

output_df_1_handle.close()
output_df_2_handle.close()



#!/usr/bin/env python3

import os
import argparse
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord
from Bio.Graphics import GenomeDiagram
from Bio.SeqFeature import FeatureLocation
from reportlab.lib import colors
from reportlab.lib.units import cm


VisGeneFlk_usage = '''
=========================== VisGeneFlk example commands ===========================

BioSAK VisGeneFlk -gene gene_01 -gbk input.gbk -len 2000 -fmt pdf
BioSAK VisGeneFlk -gene gene_01 -gbk input.gbk -len 5000 -scale 300

===================================================================================
'''


def get_flanking_region(input_gbk_file, HGT_candidate, flanking_length):

    wd, gbk_file = os.path.split(input_gbk_file)
    new_gbk_file = '%s/%s_%sbp_temp.gbk' % (wd, HGT_candidate, flanking_length)
    new_gbk_final_file = '%s/%s_%sbp.gbk' % (wd, HGT_candidate, flanking_length)

    # get flanking range of candidate
    input_gbk = SeqIO.parse(input_gbk_file, "genbank")
    new_start = 0
    new_end = 0
    contig_length = 0
    for record in input_gbk:
        contig_length = len(record.seq)
        for gene in record.features:
            # get contig length
            if gene.type == 'source':
                pass
            # get new start and end points
            elif 'locus_tag' in gene.qualifiers:
                if gene.qualifiers['locus_tag'][0] == HGT_candidate:
                    # get new start point
                    new_start = gene.location.start - flanking_length
                    if new_start < 0:
                        new_start = 0
                    # get new end point
                    new_end = gene.location.end + flanking_length
                    if new_end > contig_length:
                        new_end = contig_length

    # get genes within flanking region
    keep_gene_list = []
    input_gbk = SeqIO.parse(input_gbk_file, "genbank")
    for record in input_gbk:
        for gene in record.features:
            if 'locus_tag' in gene.qualifiers:
                if (gene.location.start < new_start) and (gene.location.end >= new_start):
                    keep_gene_list.append(gene.qualifiers['locus_tag'][0])
                    new_start = gene.location.start
                elif (gene.location.start > new_start) and (gene.location.end < new_end):
                    keep_gene_list.append(gene.qualifiers['locus_tag'][0])
                elif (gene.location.start <= new_end) and (gene.location.end > new_end):
                    keep_gene_list.append(gene.qualifiers['locus_tag'][0])
                    new_end = gene.location.end

    # remove genes not in flanking region from gbk file
    input_gbk = SeqIO.parse(input_gbk_file, "genbank")
    new_gbk = open(new_gbk_file, 'w')
    for record in input_gbk:
        new_record_features = []
        for gene in record.features:
            if gene.type == 'source':
                new_record_features.append(gene)
            elif 'locus_tag' in gene.qualifiers:
                if gene.qualifiers['locus_tag'][0] in keep_gene_list:
                    new_record_features.append(gene)
        record.features = new_record_features
        SeqIO.write(record, new_gbk, 'genbank')
    new_gbk.close()

    # remove sequences not in flanking region
    new_gbk_full_length = SeqIO.parse(new_gbk_file, "genbank")
    new_gbk_final = open(new_gbk_final_file, 'w')
    for record in new_gbk_full_length:

        # get new sequence
        new_seq = record.seq[new_start:new_end]
        new_contig_length = len(new_seq)
        new_record = SeqRecord(new_seq, id=record.id, name=record.name,
                               description=record.description,
                               annotations=record.annotations)

        # get new location
        new_record_features_2 = []
        for gene in record.features:
            if gene.type == 'source':
                gene_location_new = ''
                if gene.location.strand == 1:
                    gene_location_new = FeatureLocation(0, new_contig_length, strand=+1)
                if gene.location.strand == -1:
                    gene_location_new = FeatureLocation(0, new_contig_length, strand=-1)
                gene.location = gene_location_new
                new_record_features_2.append(gene)
            elif 'locus_tag' in gene.qualifiers:
                gene_location_new = ''
                if gene.location.strand == 1:
                    if gene.location.start - new_start < 0:
                        gene_location_new = FeatureLocation(0,
                                                            gene.location.end - new_start,
                                                            strand=+1)
                    else:
                        gene_location_new = FeatureLocation(gene.location.start - new_start,
                                                            gene.location.end - new_start,
                                                            strand=+1)
                if gene.location.strand == -1:
                    if gene.location.start - new_start < 0:
                        gene_location_new = FeatureLocation(0,
                                                            gene.location.end - new_start,
                                                            strand=-1)
                    else:
                        gene_location_new = FeatureLocation(gene.location.start - new_start,
                                                            gene.location.end - new_start,
                                                            strand=-1)
                gene.location = gene_location_new
                new_record_features_2.append(gene)
        new_record.features = new_record_features_2
        SeqIO.write(new_record, new_gbk_final, 'genbank')

    new_gbk_final.close()
    os.system('rm %s' % new_gbk_file)


def set_contig_track_features(gene_contig, gene_to_highlight, feature_set):

    # add features to feature set
    for feature in gene_contig.features:
        if feature.type == "CDS":

            # define label color
            if feature.qualifiers['locus_tag'][0] == gene_to_highlight:
                label_color = colors.red
                label_size = 10
            else:
                label_color = colors.black
                label_size = 10

            # strands
            color = None
            label_angle = 0
            if feature.location.strand == 1:
                label_angle = 45
                color = colors.lightblue
            elif feature.location.strand == -1:
                label_angle = -225
                color = colors.lightgreen

            # add feature
            feature_set.add_feature(feature, color=color, label=True, sigil='ARROW', arrowshaft_height=0.5, arrowhead_length=0.4, label_color=label_color, label_size=label_size, label_angle=label_angle, label_position="middle")


def VisGeneFlk(args):

    gene_id         = args['gene']
    input_gbk       = args['gbk']
    flanking_length = args['len']
    plot_scale      = args['scale']
    plot_fmt        = args['fmt']

    plot_wd                  = '%s_flk%s_wd'    % (gene_id, flanking_length)
    gbk_subset_located_seq   = '%s/%s.gbk'      % (plot_wd, gene_id)
    gbk_subset_flanking_gene = '%s/%s_%sbp.gbk' % (plot_wd, gene_id, flanking_length)
    plot_file                = '%s_flk%sbp.%s'  % (gene_id, flanking_length, plot_fmt)

    if os.path.isdir(plot_wd) is False:
        os.mkdir(plot_wd)
    else:
        os.system('rm -r %s' % plot_wd)
        os.mkdir(plot_wd)

    dict_value_list = []
    for seq_record in SeqIO.parse(input_gbk, 'genbank'):
        for gene_feature in seq_record.features:
            if 'locus_tag' in gene_feature.qualifiers:
                if gene_id in gene_feature.qualifiers["locus_tag"]:
                    dict_value_list.append([gene_id, int(gene_feature.location.start), int(gene_feature.location.end), gene_feature.location.strand, len(seq_record.seq)])
                    SeqIO.write(seq_record, gbk_subset_located_seq, 'genbank')
                    get_flanking_region(gbk_subset_located_seq, gene_id, flanking_length)

    # get the distance of the gene to contig ends
    gene_1_left_len = dict_value_list[0][1]
    gene_1_right_len = dict_value_list[0][4] - dict_value_list[0][2]

    # read in gbk file
    sequence_record = SeqIO.read(gbk_subset_flanking_gene, "genbank")

    # create an empty diagram
    diagram = GenomeDiagram.Diagram()
    plot_len_cm = len(sequence_record)/plot_scale

    # add tracks to diagram
    track_footnote = '%s (left %sbp, right %sbp)' % (sequence_record.name, gene_1_left_len, gene_1_right_len)
    track_footnote = sequence_record.name
    seq_track = diagram.new_track(1, name=track_footnote, greytrack=True,
                                  greytrack_labels=1, greytrack_font='Helvetica', greytrack_fontsize=12,
                                  height=0.35, start=0, end=len(sequence_record),
                                  scale=True, scale_fontsize=6, scale_ticks=1,
                                  scale_smalltick_interval=10000, scale_largetick_interval=10000)

    # create blank feature set and add gene features to it
    feature_set = seq_track.new_set(type='feature')
    set_contig_track_features(sequence_record, gene_id, feature_set)

    # draw and export
    diagram.draw(format='linear', orientation='landscape', pagesize=(20*cm, plot_len_cm*cm), fragments=1, start=0, end=len(sequence_record))
    diagram.write(plot_file, plot_fmt)


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-gene',    required=True,                          help='gene id')
    parser.add_argument('-gbk',     required=True,                          help='gbk file')
    parser.add_argument('-len',     required=True, type=int,                help='length (in bp) of flanking sequences to plot')
    parser.add_argument('-scale',   required=False, type=int, default=200,  help='scale for plotting, default: 200bp per cm')
    parser.add_argument('-fmt',     required=False, default='svg',          help='output format (svg or pdf), default: svg')

    args = vars(parser.parse_args())
    VisGeneFlk(args)



import os
import shutil
from Bio import SeqIO
from Bio.Seq import Seq

def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)
    os.mkdir(folder_to_create)


def sep_reads_by_barcode(fastq_in, sample_to_barcode_file, direction):

    # get barcode to sample dict
    barcode_to_sample_dict = {}
    for barcode in open(sample_to_barcode_file):

        barcode_split = barcode.strip().split(',')
        sample_id = barcode_split[0]
        barcode_seq = barcode_split[1]

        if direction == 'F':
            barcode_to_sample_dict[barcode_seq] = sample_id

        if direction == 'R':
            barcode_seq_reverse_complement = str(Seq(barcode_seq).reverse_complement())
            barcode_to_sample_dict[barcode_seq_reverse_complement] = sample_id


    for each_read in SeqIO.parse(fastq_in, 'fastq'):
        fastq_in_basename = '.'.join(fastq_in.split('.')[:-1])
        each_read_seq = str(each_read.seq)
        each_read_seq_first_10bp = each_read_seq[:15]

        current_barcode = ''
        current_read_sample = ''
        for barcode in barcode_to_sample_dict:
            if barcode in each_read_seq_first_10bp:
                current_barcode = barcode
                current_read_sample = barcode_to_sample_dict[barcode]

        if (current_barcode != '') and (current_read_sample != ''):
            barcode_pos_start = each_read_seq.index(current_barcode)
            barcode_pos_end = barcode_pos_start + len(current_barcode)
            read_without_barcode = each_read_seq[barcode_pos_end:]
            each_read_qual_str = each_read.format("fastq").split('\n')[3]
            read_qual_str_without_barcode = each_read_qual_str[barcode_pos_end:]

            # write out reads without barcode to corresponding files
            pwd_output_file = '%s/%s_%s.fastq' % (output_folder, fastq_in_basename, current_read_sample)
            output_file_handle = open(pwd_output_file, 'a')
            output_file_handle.write('@%s\n' % each_read.description)
            output_file_handle.write('%s\n' % read_without_barcode)
            output_file_handle.write('+\n')
            output_file_handle.write('%s\n' % read_qual_str_without_barcode)
            output_file_handle.close()


# define input and output files
wd = '/Users/songweizhi/Desktop/1-1_raw_data'
sample_to_barcode_file =    'sample_to_barcode.txt'
r1_fastq =                  'R1_total.fastq'
r2_fastq =                  'R2_total.fastq'
output_folder =             'separated_reads'


# go to working directory
os.chdir(wd)

# create output folder
force_create_folder(output_folder)

# separate reads by barcode
#sep_reads_by_barcode(r1_fastq, sample_to_barcode_file, 'F')
sep_reads_by_barcode(r2_fastq, sample_to_barcode_file, 'R')



import os
from pathlib import Path


db_var = "ENRICHM_DB"
if db_var in os.environ:
    DATABASE_DIR = os.environ[db_var]
else:
    DATA_PATH = str(Path.home())
    DATABASE_DIR = os.path.join(DATA_PATH, 'databases')




#!/usr/bin/python3
import os
import re
#import dendropy
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC
from Bio.SeqRecord import SeqRecord


# NEEDS, hmmer3, mafft and fasttree
# Call with:    1) Path to folder containing all prokka output folders
#               2) Path to its temporary output folder
#               3) Path to the phylo.hmm file
#               4) Path to write the final alignment and tree


def export_dna_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.unambiguous_dna)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def treeMaker(path_to_prokka, path_to_hmm):

    # Tests for presence of the tmp folder and deletes it
    tmp_folder = 'hmm_tmp'
    if os.path.exists(tmp_folder):
        os.system('rm -r ' + tmp_folder)
    os.mkdir(tmp_folder)

    # List all prokka dirs in the target folder
    prokka_files = [i for i in os.listdir(path_to_prokka) if os.path.isdir(path_to_prokka + '/' + i)]
    print('Detected %i input genomes' % len(prokka_files))

    # Running hmmsearch on each file
    print('Running hmmsearch...')
    for f in prokka_files:
        # call hmmsearch
        #os.system('hmmsearch -o /dev/null --domtblout %s/%s_hmmout.tbl %s %s/%s/%s.faa' % (tmp_folder, f, path_to_hmm, path_to_prokka, f, f))
        os.system('/Users/songweizhi/Softwares/hmmer/hmmer-3.1b2-macosx-intel/binaries/hmmsearch -o /dev/null --domtblout %s/%s_hmmout.tbl %s %s/%s/%s.faa' % (tmp_folder, f, path_to_hmm, path_to_prokka, f, f))

        # Reading the protein file in a dictionary
        proteinSequence = {}
        for seq_record in SeqIO.parse('%s/%s/%s.faa' % (path_to_prokka, f, f), 'fasta'):
            proteinSequence[seq_record.id] = str(seq_record.seq)

        # Reading the hmmersearch table/extracting the protein part found beu hmmsearch out of the protein/Writing each protein sequence that was extracted to a fasta file (one for each hmm in phylo.hmm
        proteinName = ''
        for line in open('%s/%s_hmmout.tbl' % (tmp_folder, f), 'r'):
            if line[0] != "#":
                line = re.sub('\s+', ' ', line) # ro remove redundant spaces
                splitLine = line.split(' ')
                if proteinName != splitLine[3]:
                    proteinName = splitLine[3]
                    file_out = open(tmp_folder + '/' + proteinName + '.fasta', 'a+')
                    x1 = int(splitLine[17]) - 1
                    x2 = int(splitLine[18])
                    export_dna_record(proteinSequence[splitLine[0]][x1: x2], f, '', file_out)
                    file_out.close()

    # Call mafft to align all single fasta files with hmms
    files = os.listdir(tmp_folder)
    fastaFiles = [i for i in files if i.endswith('.fasta')]
    print('Running mafft...')
    for f in fastaFiles:
        fastaFile1 = '%s/%s' % (tmp_folder, f)
        fastaFile2 = fastaFile1.replace('.fasta', '_aligned.fasta')
        os.system('mafft --quiet --maxiterate 1000 --globalpair ' + fastaFile1 + ' > ' + fastaFile2 + ' ; rm ' + fastaFile1)

    # concatenating the single alignments
    # create the dictionary
    print('Concatenating alignments...')
    concatAlignment = {}
    for element in prokka_files:
        concatAlignment[element] = ''

    # Reading all single alignment files and append them to the concatenated alignment
    files = os.listdir(tmp_folder)
    fastaFiles = [i for i in files if i.endswith('.fasta')]
    for f in fastaFiles:
        fastaFile = tmp_folder + '/' + f
        proteinSequence = {}
        alignmentLength = 0
        for seq_record_2 in SeqIO.parse(fastaFile, 'fasta'):
            proteinName =seq_record_2.id
            proteinSequence[proteinName] = str(seq_record_2.seq)
            alignmentLength = len(proteinSequence[proteinName])

        for element in prokka_files:
            if element in proteinSequence.keys():
                concatAlignment[element] += proteinSequence[element]
            else:
                concatAlignment[element] += '-' * alignmentLength

    # writing alignment to file
    file_out = open('./alignment.fasta', 'w')
    for element in prokka_files:
        file_out.write('>' + element + '\n' + concatAlignment[element] + '\n')
    file_out.close()

    # calling fasttree for tree calculation
    print('Running fasttree...')
    os.system('fasttree -quiet alignment.fasta > phylogenticTree.phy')

    # Decomment the two following lines if tree is rooted but should be unrooted
    #phyloTree = dendropy.Tree.get(path='phylogenticTree.phy', schema='newick', rooting='force-unrooted')
    #dendropy.Tree.write_to_path(phyloTree, 'phylogenticTree_unrooted.phy', 'newick')
    os.system('rm -r ' + tmp_folder)
    print('Tree exported to phylogenticTree.phy')


os.chdir('/Users/songweizhi/Desktop/hmmTree')
treeMaker('prokka', 'phylo.hmm')



"""Convert a GFF and associated FASTA file into GenBank format.

Usage:
gff_to_genbank.py <GFF annotation file> <FASTA sequence file>
"""
import sys
import os

from Bio import SeqIO
from Bio.Alphabet import generic_dna
from BCBio import GFF

def main(gff_file, fasta_file):
    out_file = "%s.gb" % os.path.splitext(gff_file)[0]
    fasta_input = SeqIO.to_dict(SeqIO.parse(fasta_file, "fasta", generic_dna))
    gff_iter = GFF.parse(gff_file, fasta_input)
    SeqIO.write(gff_iter, out_file, "genbank")

if __name__ == "__main__":
    main(*sys.argv[1:])



import os
import glob
import shutil
import argparse
from Bio import SeqIO
from time import sleep
from datetime import datetime
from BioSAK.BioSAK_config import config_dict


link_16S_MAG_usage = '''
=========================================== link_16S_MAG example commands ===========================================

1. rename paired reads
link_16S_MAG assumes the id of paired reads in a format of XXXX.1 and XXXX.2. The only difference of their names is 
the last character. "BioSAK rename_reads_for_Reago" do this job.
 
BioSAK link_16S_MAG -p Test -r1 ST13_R1.fasta -r2 ST13_R2.fasta -m ST13_16S_seqs.fa -mag ST13_ctgs.fasta -t 4
BioSAK link_16S_MAG -p Test -r1 ST13_R1.fasta -r2 ST13_R2.fasta -m ST13_16S_seqs.fa -mag ST13_MAGs -x fa -t 4

=====================================================================================================================
'''

def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)
    os.mkdir(folder_to_create)


def report_and_log(message_for_report, log_file, keep_quiet):

    time_format = '[%Y-%m-%d %H:%M:%S]'
    with open(log_file, 'a') as log_handle:
        log_handle.write('%s %s\n' % ((datetime.now().strftime(time_format)), message_for_report))

    if keep_quiet is False:
        print('%s %s' % ((datetime.now().strftime(time_format)), message_for_report))


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def cigar_splitter(cigar):

    # get the position of letters
    letter_pos_list = []
    n = 0
    for each_element in cigar:
        if each_element.isalpha() is True:
            letter_pos_list.append(n)
        n += 1

    # split cigar
    index = 0
    cigar_splitted = []
    while index <= len(letter_pos_list) - 1:
        if index == 0:
            cigar_splitted.append(cigar[:(letter_pos_list[index] + 1)])
        else:
            cigar_splitted.append(cigar[(letter_pos_list[index - 1] + 1):(letter_pos_list[index] + 1)])
        index += 1

    return cigar_splitted


def remove_unmapped_reads(sam_file, sam_file_only_mapped):

    sam_file_only_mapped_handle = open(sam_file_only_mapped, 'w')

    for each_read in open(sam_file):
        if not each_read.startswith('@'):
            ref_id = each_read.strip().split('\t')[2]
            cigar = each_read.strip().split('\t')[5]

            if (ref_id != '*') and (cigar != '*'):
                sam_file_only_mapped_handle.write(each_read)

    sam_file_only_mapped_handle.close()


def blast_results_to_dict(blastn_results, iden_cutoff, query_cov_cutoff):
    query_to_subject_list_dict = {}

    for blast_hit in open(blastn_results):
        blast_hit_split = blast_hit.strip().split('\t')
        query = blast_hit_split[0]
        subject = blast_hit_split[1]
        subject_with_prefix = 'GenomicSeq__%s' % subject
        iden = float(blast_hit_split[2])
        align_len = int(blast_hit_split[3])
        query_len = int(blast_hit_split[12])
        coverage_q = float(align_len) * 100 / float(query_len)
        if (iden >= iden_cutoff) and (coverage_q >= query_cov_cutoff):
            if query not in query_to_subject_list_dict:
                query_to_subject_list_dict[query] = [subject_with_prefix]
            else:
                query_to_subject_list_dict[query].append(subject_with_prefix)

    return query_to_subject_list_dict


def stats_dict_to_sankey_file_in(clipping_stats_dict, paired_stats_dict, value_type, signal_cutoff, sankey_file_in_clipping, sankey_file_in_paired, sankey_file_in_combined):

    file_header = ''
    if value_type == 'reads_number':
        file_header = 'GenomicSeq,MarkerGene,Number\n'
    if value_type == 'reads_length':
        file_header = 'GenomicSeq,MarkerGene,Length_bp\n'

    contig_to_plot_paired = set()
    contig_to_plot_clipping = set()
    contig_to_plot_combined = set()

    # prepare input file for plot of clipping mapped reads
    sankey_file_in_combined_handle = open(sankey_file_in_combined, 'w')
    sankey_file_in_clipping_handle = open(sankey_file_in_clipping, 'w')
    sankey_file_in_combined_handle.write(file_header)
    sankey_file_in_clipping_handle.write(file_header)
    for each_clipping in clipping_stats_dict:
        if clipping_stats_dict[each_clipping] >= signal_cutoff:
            sankey_file_in_clipping_handle.write('%s,%s\n' % (','.join(each_clipping.split('_|_')), clipping_stats_dict[each_clipping]))
            sankey_file_in_combined_handle.write('%s,%s\n' % (','.join(each_clipping.split('_|_')), clipping_stats_dict[each_clipping]))
            contig_to_plot_clipping.add(each_clipping.split('_|_')[1])
            contig_to_plot_combined.add(each_clipping.split('_|_')[1])
    sankey_file_in_clipping_handle.close()

    # prepare input file for plot of paired reads
    sankey_file_in_paired_handle = open(sankey_file_in_paired, 'w')
    sankey_file_in_paired_handle.write(file_header)
    for each_paired in paired_stats_dict:
        if paired_stats_dict[each_paired] >= signal_cutoff:
            sankey_file_in_paired_handle.write('%s,%s\n' % (','.join(each_paired.split('_|_')), paired_stats_dict[each_paired]))
            sankey_file_in_combined_handle.write('%s,%s\n' % (','.join(each_paired.split('_|_')), paired_stats_dict[each_paired]))
            contig_to_plot_paired.add(each_paired.split('_|_')[1])
            contig_to_plot_combined.add(each_paired.split('_|_')[1])
    sankey_file_in_paired_handle.close()
    sankey_file_in_combined_handle.close()

    return len(contig_to_plot_paired), len(contig_to_plot_clipping), len(contig_to_plot_combined)


def rename_seq(ctg_file_in, ctg_file_out, prefix):

    ctg_file_out_handle = open(ctg_file_out, 'w')
    for Seq_record in SeqIO.parse(ctg_file_in, 'fasta'):
        Seq_record.id = '%s___%s' % (prefix, Seq_record.id)
        SeqIO.write(Seq_record, ctg_file_out_handle, 'fasta')
    ctg_file_out_handle.close()


def link_16S_MAG(args, config_dict):

    ###################################################### file in/out #####################################################

    # file in
    output_prefix                       = args['p']
    reads_file_r1                       = args['r1']
    reads_file_r2                       = args['r2']
    genomic_assemblies                  = args['g']
    mag_folder                          = args['mag']
    mag_file_extension                  = args['x']
    marker_gene_seqs                    = args['m']
    min_cigar_M                         = args['cigarM']
    min_cigar_S                         = args['cigarS']
    iden_cutoff                         = args['iden']
    cov_cutoff                          = args['cov']
    min_num_to_plot                     = args['num']
    min_len_to_plot                     = args['len']
    run_mapping                         = args['run_mapping']
    run_blastn                          = args['run_blastn']
    force_overwrite                     = args['force']
    keep_quiet                          = args['quiet']
    num_threads                         = args['t']
    keep_temp                           = args['tmp']

    pwd_plot_sankey_R                   = config_dict['get_sankey_plot_R']
    pwd_bowtie2_exe                     = config_dict['bowtie2']
    pwd_bowtie2_build_exe               = config_dict['bowtie2_build']
    pwd_makeblastdb_exe                 = config_dict['makeblastdb']
    pwd_blastn_exe                      = config_dict['blastn']


    ################################################ check dependencies ################################################


    ############################################# create working directory #############################################

    # create working directory
    working_directory = '%s_link_16S_and_MAG_wd' % output_prefix
    if (os.path.isdir(working_directory) is True) and (force_overwrite is False):
        print('Working directory detected, program exited!')
        exit()
    else:
        force_create_folder(working_directory)


    ######################## check genomic sequence type and prepare files for making blast db #########################

    blast_db         = ''
    genomic_seq_type = ''  # ctg or mag

    # check the type of input genomic sequences
    if (genomic_assemblies is not None) and (mag_folder is None):
        genomic_seq_type = 'ctg'
        metagenomic_assemblies_file_path, metagenomic_assemblies_file_basename, metagenomic_assemblies_file_extension = sep_path_basename_ext(genomic_assemblies)
        blast_db_dir = '%s/%s_%s_blast_db' % (working_directory, output_prefix, metagenomic_assemblies_file_basename)
        blast_db     = '%s/%s%s'           % (blast_db_dir, metagenomic_assemblies_file_basename, metagenomic_assemblies_file_extension)
        os.mkdir(blast_db_dir)
        os.system('cp %s %s/' % (genomic_assemblies, blast_db_dir))

    elif (genomic_assemblies is None) and (mag_folder is not None):
        genomic_seq_type = 'mag'
        mag_folder_name     = mag_folder.split('/')[-1]
        blast_db_dir        = '%s/%s_blast_db'            % (working_directory, mag_folder_name)
        renamed_mag_folder  = '%s/%s_blast_db/%s_renamed' % (working_directory, mag_folder_name, mag_folder_name)
        os.mkdir(blast_db_dir)
        os.mkdir(renamed_mag_folder)

        # get input mag file list
        mag_file_re = '%s/*%s' % (mag_folder, mag_file_extension)
        mag_file_list = [os.path.basename(file_name) for file_name in glob.glob(mag_file_re)]
        if len(mag_file_list) == 0:
            print('No MAG detected, program exited!')
            exit()

        # add mag id to its sequences
        for mag_in in mag_file_list:
            pwd_mag_in      = '%s/%s' % (mag_folder, mag_in)
            pwd_mag_renamed = '%s/%s' % (renamed_mag_folder, mag_in)
            mag_basename    = '.'.join(mag_in.split('.')[:-1])
            rename_seq(pwd_mag_in, pwd_mag_renamed, mag_basename)

        # combine renamed MAGs
        blast_db = '%s/%s_combined.fa' % (blast_db_dir, mag_folder_name)
        os.system('cat %s/*%s > %s' % (renamed_mag_folder, mag_file_extension, blast_db))

    else:
        print('Please provide genomic sequences either as raw assemblies (-g) or as MAGs (-mag)')
        exit()


    ########################################### define folder and file name ############################################

    marker_gene_seqs_file_path, marker_gene_seqs_file_basename, marker_gene_seqs_file_extension = sep_path_basename_ext(marker_gene_seqs)

    pwd_log_file                                = '%s/%s_%s.log.txt'                                                        % (working_directory, output_prefix, datetime.now().strftime('%Y-%m-%d_%Hh-%Mm-%Ss_%f'))
    bowtie_index_dir                            = '%s/%s_%s_index'                                                          % (working_directory, output_prefix, marker_gene_seqs_file_basename)
    pwd_samfile                                 = '%s/%s.sam'                                                               % (working_directory, marker_gene_seqs_file_basename)
    clipping_reads_matched_part                 = '%s/%s_clipping_matched_part.txt'                                         % (working_directory, output_prefix)
    clipping_reads_not_matched_part_seq         = '%s/%s_clipping_not_matched_part_seq.fasta'                               % (working_directory, output_prefix)
    clipping_reads_not_matched_part_seq_blastn  = '%s/%s_clipping_not_matched_part_seq_blast.txt'                           % (working_directory, output_prefix)
    clipping_reads_match_profile                = '%s/%s_match_profile_clipping.txt'                                        % (working_directory, output_prefix)
    extracted_reads_file                        = '%s/%s_paired_unmapped.fasta'                                             % (working_directory, output_prefix)
    extracted_reads_blastn                      = '%s/%s_paired_unmapped_blast.txt'                                         % (working_directory, output_prefix)
    paired_reads_match_profile                  = '%s/%s_match_profile_paired.txt'                                          % (working_directory, output_prefix)
    clipping_stats_num                          = '%s/%s_plot_cigarM%s_cigarS%s_iden%s_cov%s_MinNum%s_clipping.txt'         % (working_directory, output_prefix, min_cigar_M, min_cigar_S, iden_cutoff, cov_cutoff, min_num_to_plot)
    paired_stats_num                            = '%s/%s_plot_cigarM%s_cigarS%s_iden%s_cov%s_MinNum%s_paired.txt'           % (working_directory, output_prefix, min_cigar_M, min_cigar_S, iden_cutoff, cov_cutoff, min_num_to_plot)
    combined_stats_num                          = '%s/%s_plot_cigarM%s_cigarS%s_iden%s_cov%s_MinNum%s_combined.txt'         % (working_directory, output_prefix, min_cigar_M, min_cigar_S, iden_cutoff, cov_cutoff, min_num_to_plot)
    clipping_stats_len                          = '%s/%s_plot_cigarM%s_cigarS%s_iden%s_cov%s_MinLen%sbp_clipping.txt'       % (working_directory, output_prefix, min_cigar_M, min_cigar_S, iden_cutoff, cov_cutoff, min_len_to_plot)
    paired_stats_len                            = '%s/%s_plot_cigarM%s_cigarS%s_iden%s_cov%s_MinLen%sbp_paired.txt'         % (working_directory, output_prefix, min_cigar_M, min_cigar_S, iden_cutoff, cov_cutoff, min_len_to_plot)
    combined_stats_len                          = '%s/%s_plot_cigarM%s_cigarS%s_iden%s_cov%s_MinLen%sbp_combined.txt'       % (working_directory, output_prefix, min_cigar_M, min_cigar_S, iden_cutoff, cov_cutoff, min_len_to_plot)


    ######################################## map reads to marker gene sequences ########################################

    if run_mapping is True:

        # copy marker gene sequence file to index folder
        report_and_log(('Indexing marker gene sequences for mapping'), pwd_log_file, keep_quiet)
        os.mkdir(bowtie_index_dir)
        os.system('cp %s %s/' % (marker_gene_seqs, bowtie_index_dir))

        # run mapping
        report_and_log(('Mapping reads to marker gene sequences'), pwd_log_file, keep_quiet)

        sleep(1)
        report_and_log(('If you see any warning messages starting with "Use of uninitialized value" during bowtie mapping, please ignore them :)'), pwd_log_file, keep_quiet)
        sleep(1)

        bowtie2_index_ref_cmd = '%s -f %s/%s%s %s/%s --quiet --threads %s' % (pwd_bowtie2_build_exe, bowtie_index_dir, marker_gene_seqs_file_basename, marker_gene_seqs_file_extension, bowtie_index_dir, marker_gene_seqs_file_basename, num_threads)
        bowtie2_mapping_cmd   = '%s -x %s/%s -1 %s -2 %s -S %s -f --local --no-unal --quiet --threads %s' % (pwd_bowtie2_exe,  bowtie_index_dir, marker_gene_seqs_file_basename, reads_file_r1, reads_file_r2, pwd_samfile, num_threads)
        os.system(bowtie2_index_ref_cmd)
        os.system(bowtie2_mapping_cmd)

        sleep(1)
        report_and_log(('If you see any warning messages starting with "Use of uninitialized value" during bowtie mapping, please ignore them :)'), pwd_log_file, keep_quiet)
        sleep(1)


    ##################################################### extract reads ####################################################

    report_and_log(('Extract unmapped paired reads and unmapped part of clipping mapped reads'), pwd_log_file, keep_quiet)

    # export clipping mapped reads and perfectly mapped reads
    clipping_mapped_reads_list = set()
    clipping_reads_mapped_part_dict = {}
    perfectly_mapped_reads_dict = {}
    clipping_reads_matched_part_handle = open(clipping_reads_matched_part, 'w')
    clipping_reads_not_matched_part_seq_handle = open(clipping_reads_not_matched_part_seq, 'w')
    for each_read in open(pwd_samfile):

        if not each_read.startswith('@'):
            each_read_split = each_read.strip().split('\t')
            read_id = each_read_split[0]
            read_id_base = '.'.join(read_id.split('.')[:-1])
            read_strand = read_id.split('.')[-1]
            ref_id = each_read_split[2]
            ref_id_with_prefix = 'MarkerGene__%s' % each_read_split[2]
            ref_pos = int(each_read_split[3])
            cigar = each_read_split[5]
            read_seq = each_read_split[9]
            cigar_splitted  = cigar_splitter(cigar)

            # for perfectly mapped reads
            if ('M' in cigar) and (len(cigar_splitted) == 1):
                if read_id_base not in perfectly_mapped_reads_dict:
                    perfectly_mapped_reads_dict[read_id_base] = {read_strand:[ref_id_with_prefix]}
                else:
                    if read_strand not in perfectly_mapped_reads_dict[read_id_base]:
                        perfectly_mapped_reads_dict[read_id_base][read_strand] = [ref_id_with_prefix]
                    else:
                        perfectly_mapped_reads_dict[read_id_base][read_strand].append(ref_id_with_prefix)

            # for clipping mapped reads
            if ('S' in cigar) and (len(cigar_splitted) == 2):
                cigar_M_len = 0
                cigar_S_len = 0
                split_pos = 0
                if cigar_splitted[0][-1] == 'M':
                    cigar_M_len = int(cigar_splitted[0][:-1])
                    cigar_S_len = int(cigar_splitted[1][:-1])
                    split_pos = ref_pos + cigar_M_len
                if cigar_splitted[1][-1] == 'M':
                    cigar_M_len = int(cigar_splitted[1][:-1])
                    cigar_S_len = int(cigar_splitted[0][:-1])
                    split_pos = ref_pos

                if (cigar_M_len >= min_cigar_M) and (cigar_S_len >= min_cigar_S):
                    read_seq_left = read_seq[: int(cigar_splitted[0][:-1])]
                    read_seq_right = read_seq[-int(cigar_splitted[1][:-1]):]

                    if cigar_splitted[0][-1] == 'M':
                        clipping_reads_matched_part_handle.write('%s_l\t%s\t%s\n' % (read_id, ref_id, split_pos))
                        clipping_reads_not_matched_part_seq_handle.write('>%s_r\n' % read_id)
                        clipping_reads_not_matched_part_seq_handle.write(read_seq_right + '\n')
                        if ('%s_l' % read_id) not in clipping_reads_mapped_part_dict:
                            clipping_reads_mapped_part_dict[('%s_l' % read_id)] = [ref_id_with_prefix]
                        else:
                            clipping_reads_mapped_part_dict[('%s_l' % read_id)].append(ref_id_with_prefix)

                    if cigar_splitted[1][-1] == 'M':
                        clipping_reads_matched_part_handle.write('%s_r\t%s\t%s\n' % (read_id, ref_id, split_pos))
                        clipping_reads_not_matched_part_seq_handle.write('>%s_l\n' % read_id)
                        clipping_reads_not_matched_part_seq_handle.write(read_seq_left + '\n')
                        if ('%s_r' % read_id) not in clipping_reads_mapped_part_dict:
                            clipping_reads_mapped_part_dict[('%s_r' % read_id)] = [ref_id_with_prefix]
                        else:
                            clipping_reads_mapped_part_dict[('%s_r' % read_id)].append(ref_id_with_prefix)

                    clipping_mapped_reads_list.add(read_id)
    clipping_reads_not_matched_part_seq_handle.close()

    perfectly_mapped_read_singleton_dict = {}
    for perfectly_mapped_read in perfectly_mapped_reads_dict:
        current_value = perfectly_mapped_reads_dict[perfectly_mapped_read]
        if len(current_value) == 1:
            perfectly_mapped_read_singleton_dict[perfectly_mapped_read] = current_value

    # get the id of paired reads to extract
    solely_perfectly_mapped_reads_r1_test = []
    solely_perfectly_mapped_reads_r2_test = []
    for perfectly_mapped_read in perfectly_mapped_read_singleton_dict:
        current_value = perfectly_mapped_read_singleton_dict[perfectly_mapped_read]
        strand = list(current_value.keys())[0]
        if strand == '1':
            r2_to_extract = '%s.2' % perfectly_mapped_read
            if r2_to_extract not in clipping_mapped_reads_list:
                solely_perfectly_mapped_reads_r2_test.append(r2_to_extract)
        if strand == '2':
            r1_to_extract = '%s.1' % perfectly_mapped_read
            if r1_to_extract not in clipping_mapped_reads_list:
                solely_perfectly_mapped_reads_r1_test.append(r1_to_extract)

    # extract reads
    if run_mapping is True:
        extracted_reads_file_handle = open(extracted_reads_file, 'w')
        for read_r1 in SeqIO.parse(reads_file_r1, 'fasta'):
            if read_r1.id in solely_perfectly_mapped_reads_r1_test:
                extracted_reads_file_handle.write('>%s\n' % read_r1.id)
                extracted_reads_file_handle.write('%s\n' % read_r1.seq)
        for read_r2 in SeqIO.parse(reads_file_r2, 'fasta'):
            if read_r2.id in solely_perfectly_mapped_reads_r2_test:
                extracted_reads_file_handle.write('>%s\n' % read_r2.id)
                extracted_reads_file_handle.write('%s\n' % read_r2.seq)
        extracted_reads_file_handle.close()

    # store reads length into dict
    reads_len_dict = {}
    for read_r1 in SeqIO.parse(reads_file_r1, 'fasta'):
        reads_len_dict[read_r1.id] = len(read_r1.seq)
    for read_r2 in SeqIO.parse(reads_file_r2, 'fasta'):
        reads_len_dict[read_r2.id] = len(read_r2.seq)


    ############################# run blast between extracted reads and metagenomic assemblies #############################

    # run blast
    if run_blastn is True:

        # copy genomic assemblies to blast db folder

        # run blastn
        blast_parameters    = '-evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads %s' % num_threads
        makeblastdb_cmd     = '%s -in %s -dbtype nucl -parse_seqids -logfile /dev/null' % (pwd_makeblastdb_exe, blast_db)
        blastn_cmd_paired   = '%s -query %s -db %s -out %s %s' % (pwd_blastn_exe, extracted_reads_file, blast_db, extracted_reads_blastn, blast_parameters)
        blastn_cmd_clipping = '%s -query %s -db %s -out %s %s' % (pwd_blastn_exe, clipping_reads_not_matched_part_seq, blast_db, clipping_reads_not_matched_part_seq_blastn, blast_parameters)

        report_and_log(('Making blastn database'), pwd_log_file, keep_quiet)
        os.system(makeblastdb_cmd)

        report_and_log(('Running blastn for unmapped paired reads'), pwd_log_file, keep_quiet)
        os.system(blastn_cmd_paired)

        report_and_log(('Running blastn for unmapped parts of clipping mapped reads'), pwd_log_file, keep_quiet)
        os.system(blastn_cmd_clipping)


    ######################################### parse blast results for paired reads #########################################

    report_and_log(('Parsing blast results for paired reads'), pwd_log_file, keep_quiet)

    # filter blast results for paired and clipping mapped reads
    unmapped_paired_reads_to_ctg_dict = blast_results_to_dict(extracted_reads_blastn, iden_cutoff, cov_cutoff)
    clipping_parts_to_ctg_dict        = blast_results_to_dict(clipping_reads_not_matched_part_seq_blastn, iden_cutoff, cov_cutoff)
    paired_stats_dict_num = {}
    paired_stats_dict_len = {}
    paired_reads_match_profile_handle = open(paired_reads_match_profile, 'w')
    paired_reads_match_profile_handle.write('ID\tR1\tR2\tPairedTotalLength(bp)\n')
    for unmapped_read in unmapped_paired_reads_to_ctg_dict:

        unmapped_read_base = '.'.join(unmapped_read.split('.')[:-1])
        unmapped_read_strand = unmapped_read.split('.')[-1]

        unmapped_read_base_r1_matched = []
        unmapped_read_base_r2_matched = []
        if unmapped_read_strand == '1':
            unmapped_read_base_r1_matched = unmapped_paired_reads_to_ctg_dict[unmapped_read]
            if unmapped_read_base in perfectly_mapped_read_singleton_dict:
                unmapped_read_base_r2_matched = perfectly_mapped_read_singleton_dict[unmapped_read_base]['2']
        if unmapped_read_strand == '2':
            unmapped_read_base_r2_matched = unmapped_paired_reads_to_ctg_dict[unmapped_read]
            if unmapped_read_base in perfectly_mapped_read_singleton_dict:
                unmapped_read_base_r1_matched = perfectly_mapped_read_singleton_dict[unmapped_read_base]['1']

        if (unmapped_read_base_r1_matched != []) and (unmapped_read_base_r2_matched != []):
            for r1 in unmapped_read_base_r1_matched:
                for r2 in unmapped_read_base_r2_matched:

                    # write out to file
                    paired_total_length = reads_len_dict[unmapped_read_base + '.1'] + reads_len_dict[unmapped_read_base + '.2']
                    paired_reads_match_profile_handle.write('%s\t%s\t%s\t%s\n' % (unmapped_read_base, r1, r2, paired_total_length))

                    # store in dict
                    paired_key = '_|_'.join(sorted([r1, r2])[::-1])
                    if genomic_seq_type == 'mag':
                        paired_key = '___'.join(paired_key.split('___')[:-1])
                    if paired_key not in paired_stats_dict_len:
                        paired_stats_dict_len[paired_key] = paired_total_length
                        paired_stats_dict_num[paired_key] = 1
                    else:
                        paired_stats_dict_len[paired_key] += paired_total_length
                        paired_stats_dict_num[paired_key] += 1

    paired_reads_match_profile_handle.close()


    #################################### parse blast results for clipping mapped reads #####################################

    report_and_log(('Parsing blast results for clipping mapped reads'), pwd_log_file, keep_quiet)

    clipping_reads_match_profile_handle = open(clipping_reads_match_profile, 'w')
    clipping_reads_match_profile_handle.write('ID\tLeft\tRight\tReadLength(bp)\n')
    clipping_stats_dict_num = {}
    clipping_stats_dict_len = {}
    for clipping_mapped_read in clipping_reads_mapped_part_dict:

        clipping_mapped_read_id = clipping_mapped_read[:-2]
        mapped_part = clipping_mapped_read[-1]

        clipping_mapped_read_matches_l = []
        clipping_mapped_read_matches_r = []
        if mapped_part == 'l':
            clipping_mapped_read_matches_l = clipping_reads_mapped_part_dict[clipping_mapped_read]
            if ('%s_r' % clipping_mapped_read_id) in clipping_parts_to_ctg_dict:
                clipping_mapped_read_matches_r = clipping_parts_to_ctg_dict[('%s_r' % clipping_mapped_read_id)]
        if mapped_part == 'r':
            clipping_mapped_read_matches_r = clipping_reads_mapped_part_dict[clipping_mapped_read]
            if ('%s_l' % clipping_mapped_read_id) in clipping_parts_to_ctg_dict:
                clipping_mapped_read_matches_l = clipping_parts_to_ctg_dict[('%s_l' % clipping_mapped_read_id)]

        if (clipping_mapped_read_matches_l != []) and (clipping_mapped_read_matches_r != []):

            for l in clipping_mapped_read_matches_l:
                for r in clipping_mapped_read_matches_r:

                    # write out to file
                    clipping_reads_match_profile_handle.write('%s\t%s\t%s\t%s\n' % (clipping_mapped_read_id, l, r, reads_len_dict[clipping_mapped_read_id]))

                    # store in dict
                    clipping_key = '_|_'.join(sorted([l, r])[::-1])
                    if genomic_seq_type == 'mag':
                        clipping_key = '___'.join(clipping_key.split('___')[:-1])

                    if clipping_key not in clipping_stats_dict_len:
                        clipping_stats_dict_len[clipping_key] = reads_len_dict[clipping_mapped_read_id]
                        clipping_stats_dict_num[clipping_key] = 1
                    else:
                        clipping_stats_dict_len[clipping_key] += reads_len_dict[clipping_mapped_read_id]
                        clipping_stats_dict_num[clipping_key] += 1

    clipping_reads_match_profile_handle.close()


    ######################################################### plot #########################################################

    report_and_log(('Plotting'), pwd_log_file, keep_quiet)

    # prepare input file for sankey plot
    ctgs_to_plot_clipping_num, ctgs_to_plot_paired_num, ctgs_to_plot_combined_num = stats_dict_to_sankey_file_in(clipping_stats_dict_num, paired_stats_dict_num, 'reads_number', min_num_to_plot, clipping_stats_num, paired_stats_num, combined_stats_num)
    ctgs_to_plot_clipping_len, ctgs_to_plot_paired_len, ctgs_to_plot_combined_len = stats_dict_to_sankey_file_in(clipping_stats_dict_len, paired_stats_dict_len, 'reads_length', min_len_to_plot, clipping_stats_len, paired_stats_len, combined_stats_len)

    # get plot height
    plot_height_clipping_num = 450 if ctgs_to_plot_clipping_num <= 30 else ctgs_to_plot_clipping_num * 15
    plot_height_paired_num   = 450 if ctgs_to_plot_paired_num   <= 30 else ctgs_to_plot_paired_num   * 15
    plot_height_combined_num = 450 if ctgs_to_plot_combined_num <= 30 else ctgs_to_plot_combined_num * 15
    plot_height_clipping_len = 450 if ctgs_to_plot_clipping_len <= 30 else ctgs_to_plot_clipping_len * 15
    plot_height_paired_len   = 450 if ctgs_to_plot_paired_len   <= 30 else ctgs_to_plot_paired_len   * 15
    plot_height_combined_len = 450 if ctgs_to_plot_combined_len <= 30 else ctgs_to_plot_combined_len * 15

    # prepare commands
    cmd_sankey_clipping_num = 'Rscript %s -f %s -x %s -y %s' % (pwd_plot_sankey_R, clipping_stats_num, 600, plot_height_clipping_num)
    cmd_sankey_clipping_len = 'Rscript %s -f %s -x %s -y %s' % (pwd_plot_sankey_R, clipping_stats_len, 600, plot_height_clipping_len)
    cmd_sankey_paired_num   = 'Rscript %s -f %s -x %s -y %s' % (pwd_plot_sankey_R, paired_stats_num,   600, plot_height_paired_num)
    cmd_sankey_paired_len   = 'Rscript %s -f %s -x %s -y %s' % (pwd_plot_sankey_R, paired_stats_len,   600, plot_height_paired_len)
    cmd_sankey_combined_num = 'Rscript %s -f %s -x %s -y %s' % (pwd_plot_sankey_R, combined_stats_num, 600, plot_height_combined_num)
    cmd_sankey_combined_len = 'Rscript %s -f %s -x %s -y %s' % (pwd_plot_sankey_R, combined_stats_len, 600, plot_height_combined_len)

    # plot
    #os.system(cmd_sankey_clipping_num)
    #os.system(cmd_sankey_clipping_len)
    #os.system(cmd_sankey_paired_num)
    #os.system(cmd_sankey_paired_len)
    os.system(cmd_sankey_combined_num)
    os.system(cmd_sankey_combined_len)


    ################################################### remove tmp files ###################################################

    if keep_temp is False:

        report_and_log(('Removing temporary files'), pwd_log_file, keep_quiet)
        os.remove(pwd_samfile)
        os.remove(clipping_reads_matched_part)
        os.remove(clipping_reads_not_matched_part_seq)
        os.remove(clipping_reads_not_matched_part_seq_blastn)
        os.remove(extracted_reads_file)
        os.remove(extracted_reads_blastn)
    os.remove(clipping_stats_num)
    os.remove(paired_stats_num)
    os.remove(clipping_stats_len)
    os.remove(paired_stats_len)


    report_and_log(('Done!'), pwd_log_file, keep_quiet)


######################################################### main #########################################################

if __name__ == '__main__':

    link_16S_and_MAG_parser = argparse.ArgumentParser()

    # arguments for link_16S_and_MAG
    link_16S_and_MAG_parser.add_argument('-p',      required=True,                           help='Output prefix')
    link_16S_and_MAG_parser.add_argument('-r1',     required=True,                           help='Paired reads r1')
    link_16S_and_MAG_parser.add_argument('-r2',     required=True,                           help='Paired reads r2')
    link_16S_and_MAG_parser.add_argument('-m',      required=True,                           help='Marker gene sequences')
    link_16S_and_MAG_parser.add_argument('-g',      required=False, default=None,            help='Genomic sequences')
    link_16S_and_MAG_parser.add_argument('-mag',    required=False, default=None,            help='Metagenome-assembled-genome (MAG) folder')
    link_16S_and_MAG_parser.add_argument('-x',      required=False, default='fasta',         help='MAG file extension, default: fasta')
    link_16S_and_MAG_parser.add_argument('-iden',   required=False, type=float, default=100, help='identity cutoff, default: 100')
    link_16S_and_MAG_parser.add_argument('-cov',    required=False, type=float, default=100, help='coverage cutoff, default: 100')
    link_16S_and_MAG_parser.add_argument('-cigarM', required=False, type=int, default=50,    help='cigarM cutoff, default: 50')
    link_16S_and_MAG_parser.add_argument('-cigarS', required=False, type=int, default=50,    help='cigarS cutoff, default: 50')
    link_16S_and_MAG_parser.add_argument('-num',    required=False, type=int, default=5,     help='minimum number of reads for a link to plot, default: 5')
    link_16S_and_MAG_parser.add_argument('-len',    required=False, type=int, default=500,   help='minimum length of reads for a link to plot, default: 500')
    link_16S_and_MAG_parser.add_argument('-t',      required=False, type=int, default=1,     help='number of threads, default: 1')
    link_16S_and_MAG_parser.add_argument('-quiet',  required=False, action="store_true",     help='not report progress')
    link_16S_and_MAG_parser.add_argument('-force',  required=False, action="store_true",     help='force overwrite existing results')
    link_16S_and_MAG_parser.add_argument('-tmp',    required=False, action="store_true",     help='keep temporary files')

    args = vars(link_16S_and_MAG_parser.parse_args())
    args['run_mapping'] = True
    args['run_blastn']  = True

    link_16S_MAG(args, config_dict)


To_do = '''

1. enable multiple placement for a read?
2. where does the paired read of the clipping mapped read mapped to? (should take into consideration!!!)
3. check whether the final output is blank
4. how to incorporate the taxonomy of MAGs and 16S sequences
5. rename reads as xxx.1 and xxx.2


export PATH=/Users/songweizhi/Softwares/bowtie2:$PATH
cd /Users/songweizhi/Desktop/link_16S_MAG
python3 ~/PycharmProjects/BioSAK/BioSAK/link_16S_MAG.py -p Shan -r1 file_in/ST13_R1_P_renamed.fasta -r2 file_in/ST13_R2_P_renamed.fasta -m file_in/scaffolds.NR.min_500bp.abd.fa -g file_in/scaffold.fa -force -t 4

export PATH=/Users/songweizhi/Softwares/bowtie2:$PATH
cd /Users/songweizhi/Desktop/link_16S_MAG
python3 ~/PycharmProjects/BioSAK/BioSAK/link_16S_MAG.py -p Shan -r1 file_in/ST13_R1_P_renamed.fasta -r2 file_in/ST13_R2_P_renamed.fasta -m file_in/scaffolds.NR.min_500bp.abd.fa -mag file_in/ST13_MAGs -x fasta -force -t 4
BioSAK link_16S_MAG -p Shan -r1 file_in/ST13_R1_P_renamed.fasta -r2 file_in/ST13_R2_P_renamed.fasta -m file_in/scaffolds.NR.min_500bp.abd.fa -mag file_in/ST13_MAGs -x fasta -force -t 4

export PATH=/Users/songweizhi/Softwares/bowtie2:$PATH
cd /Users/songweizhi/Desktop/link_16S_MAG
python3 ~/PycharmProjects/BioSAK/BioSAK/link_16S_MAG.py -p Madeup -r1 madeup_file_in/reads_R1.fasta -r2 madeup_file_in/reads_R2.fasta -m madeup_file_in/16S_seq.fasta -g madeup_file_in/scaffold.fa -force -t 4 -len 1 -num 1

'''




def subset_prokaryotes_csv(GTDB_genomes, GenBank_genomes, GenBank_genomes_subset):

    num_to_full_id_dict = {}
    GTDB_genome_id_list = set()
    for each in open(GTDB_genomes):
        if not each.startswith('"ID",'):
            each_split = each.strip().split(',')
            NCBI_ID_with_version = each_split[0][1:-1]
            NCBI_ID_without_version = '.'.join(NCBI_ID_with_version.split('.')[:-1])
            NCBI_ID_no_version_only_number = NCBI_ID_without_version[3:]
            GTDB_genome_id_list.add(NCBI_ID_no_version_only_number)
            num_to_full_id_dict[NCBI_ID_no_version_only_number] = NCBI_ID_with_version

    GenBank_genomes_subset_handle = open(GenBank_genomes_subset, 'w')
    found_in_GenBank_genome_id_list = set()
    for GenBank_genome in open(GenBank_genomes):
        if not GenBank_genome.startswith('#Organism Name'):
            GenBank_genome_split = GenBank_genome.strip().split(',')
            GenBank_genome_id_with_version = GenBank_genome_split[5][1:-1]
            GenBank_genome_id_without_version = '.'.join(GenBank_genome_id_with_version.split('.')[:-1])
            GenBank_genome_id_no_version_only_number = GenBank_genome_id_without_version[3:]
            if GenBank_genome_id_no_version_only_number in GTDB_genome_id_list:
                found_in_GenBank_genome_id_list.add(GenBank_genome_id_no_version_only_number)
                GenBank_genomes_subset_handle.write(GenBank_genome)
    GenBank_genomes_subset_handle.close()

    for each in GTDB_genome_id_list:
        if each not in found_in_GenBank_genome_id_list:
            print('not found: %s' % num_to_full_id_dict[each])


GTDB_genomes           = '/Users/songweizhi/Desktop/GTDB.csv'
GenBank_genomes        = '/Users/songweizhi/Desktop/prokaryotes.csv'
GenBank_genomes_subset = '/Users/songweizhi/Desktop/prokaryotes_subset.csv'

subset_prokaryotes_csv(GTDB_genomes, GenBank_genomes, GenBank_genomes_subset)




# https://www.statsmodels.org/dev/generated/statsmodels.stats.multitest.multipletests.html
# https://campus.datacamp.com/courses/practicing-statistics-interview-questions-in-python/statistical-experiments-and-significance-testing?ex=13

from statsmodels.stats.multitest import multipletests

p_values = [0.01, 0.05, 0.10, 0.50, 0.99]

p_values_adjusted = multipletests(p_values, alpha=0.1, method='bonferroni')[1]

print(p_values_adjusted)




import statistics as stats
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt


def box_plotter_with_dots(HGT_per_Mbp_list, HGT_num_per_genome_boxplot):

    HGT_per_Mbp_list_with_class = []
    for each in HGT_per_Mbp_list:
        HGT_per_Mbp_list_with_class.append([each, 1])

    HGT_per_Mbp_list_with_class_df = pd.DataFrame(np.array(HGT_per_Mbp_list_with_class), columns=['num', 'label'])

    sns.set(style="whitegrid")  # whitegrid

    # get boxplot
    orient = 'v'
    ax = sns.boxplot(x="label", y="num", data=HGT_per_Mbp_list_with_class_df, orient=orient, width=0.5,
                     showfliers=False)
    ax = sns.stripplot(x="label", y="num", data=HGT_per_Mbp_list_with_class_df, orient=orient, color='orange', size=3,
                       jitter=0.23)
    ax.set(ylabel='Number of HGT/Mbp sequence', xlabel='Genome')

    # plt.show()
    plt.savefig(HGT_num_per_genome_boxplot, dpi=300)
    plt.close()


box_plotter_with_dots([1,2,3,4,5], '/Users/songweizhi/Desktop/test.png')



#!/usr/bin/env python

import sys

__author__ = 'weizhisong'

usage = """
####################################################################

  Usage:
  python COG_number_count.py annotation_results.txt output.txt

  COG_annotation_results format:
  G_00005	yebN	S	COG1971	Predicted membrane protein
  G_00009	PA1596	O	COG0326	Molecular chaperone, HSP90 family
  ...

  Author: Weizhi Song

####################################################################
"""

if len(sys.argv) != 3:
    print (usage)
    exit(1)

# Generate COG list for all genes:

input_file = sys.argv[1]
output_file = sys.argv[2]

annotation_results = open(input_file)
out = open(output_file, 'w')

all_cogs = []
for each_gene in annotation_results:
    cog_category = each_gene.strip().split('\t')[2]
    all_cogs.append(cog_category)


all_cogs_temp = []
for item in all_cogs:
    if len(item) == 1:
        all_cogs_temp.append(item)
    else:
        list(item)
        all_cogs_temp += list(item)

Count_A = 'A' + '\t' + str(all_cogs_temp.count('A')) + '\n'
Count_B = 'B' + '\t' + str(all_cogs_temp.count('B')) + '\n'
Count_C = 'C' + '\t' + str(all_cogs_temp.count('C')) + '\n'
Count_D = 'D' + '\t' + str(all_cogs_temp.count('D')) + '\n'
Count_E = 'E' + '\t' + str(all_cogs_temp.count('E')) + '\n'
Count_F = 'F' + '\t' + str(all_cogs_temp.count('F')) + '\n'
Count_G = 'G' + '\t' + str(all_cogs_temp.count('G')) + '\n'
Count_H = 'H' + '\t' + str(all_cogs_temp.count('H')) + '\n'
Count_I = 'I' + '\t' + str(all_cogs_temp.count('I')) + '\n'
Count_J = 'J' + '\t' + str(all_cogs_temp.count('J')) + '\n'
Count_K = 'K' + '\t' + str(all_cogs_temp.count('K')) + '\n'
Count_L = 'L' + '\t' + str(all_cogs_temp.count('L')) + '\n'
Count_M = 'M' + '\t' + str(all_cogs_temp.count('M')) + '\n'
Count_N = 'N' + '\t' + str(all_cogs_temp.count('N')) + '\n'
Count_O = 'O' + '\t' + str(all_cogs_temp.count('O')) + '\n'
Count_P = 'P' + '\t' + str(all_cogs_temp.count('P')) + '\n'
Count_Q = 'Q' + '\t' + str(all_cogs_temp.count('Q')) + '\n'
Count_R = 'R' + '\t' + str(all_cogs_temp.count('R')) + '\n'
Count_S = 'S' + '\t' + str(all_cogs_temp.count('S')) + '\n'
Count_T = 'T' + '\t' + str(all_cogs_temp.count('T')) + '\n'
Count_U = 'U' + '\t' + str(all_cogs_temp.count('U')) + '\n'
Count_V = 'V' + '\t' + str(all_cogs_temp.count('V')) + '\n'
Count_W = 'W' + '\t' + str(all_cogs_temp.count('W')) + '\n'
Count_Y = 'Y' + '\t' + str(all_cogs_temp.count('Y')) + '\n'
Count_Z = 'Z' + '\t' + str(all_cogs_temp.count('Z')) + '\n'

in_sum = Count_A + Count_B + Count_C + Count_D + Count_E + Count_F + Count_G \
         + Count_H + Count_I + Count_J + Count_K + Count_L + Count_M + Count_N \
         + Count_O + Count_P + Count_Q + Count_R + Count_S + Count_T + Count_U \
         + Count_V + Count_W + Count_Y + Count_Z

print(in_sum)
out.write(in_sum)
out.close()



import os
import argparse
from Bio import SeqIO


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def get_GC(gene_seq_file):

    seq_file_path, seq_file_basename, seq_file_extension = sep_path_basename_ext(gene_seq_file)
    output_file = '%s/%s_GC_content.txt' % (seq_file_path, seq_file_basename)

    output_file_handle = open(output_file, 'w')
    output_file_handle.write('ID\tGC(%)\n')
    for seq_record in SeqIO.parse(gene_seq_file, 'fasta'):

        seq_id = seq_record.id
        seq_sequence = str(seq_record.seq)
        seq_gc = (seq_sequence.count('G') + seq_sequence.count('g') + seq_sequence.count('C') + seq_sequence.count('c'))/len(seq_sequence)
        seq_gc = float("{0:.2f}".format(seq_gc*100))
        output_file_handle.write('%s\t%s\n' % (seq_id, seq_gc))

    output_file_handle.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser()

    parser.add_argument('-in', required=True, help='input sequence file')

    args = vars(parser.parse_args())
    gene_seq_file = args['in']

    get_GC(gene_seq_file)



#  http://stackoverflow.com/questions/14391959/heatmap-in-matplotlib-with-pcolor

import string
import matplotlib.pyplot as plt
import numpy as np

group_idens = open("/Users/weizhisong/Desktop/working_directory/0_output_3rd_average.txt")
#out = open("/Users/weizhisong/Desktop/working_directory/for_plot_in_r.txt",'w')
group_numbers = 12


def get_group_identity_dic(input_file):
    group_identity_dic = {}
    for each_line in input_file:
        each_line_split = each_line.strip().split('\t')
        group_name = each_line_split[0]
        identity = each_line_split[1]
        group_identity_dic[group_name] = identity
    return group_identity_dic


def get_identity_list(group_iden_dic, group_number):
    all_alphabet_list = list(string.ascii_uppercase)
    sub_alphabet_list = all_alphabet_list[0: group_number]
    members = []
    members_iden_str = []
    members_iden_flo = []
    #out.write (',' + '%s' % ','.join(sub_alphabet_list) + '\n') # The first title line
    #out.write ('%s' % ','.join(sub_alphabet_list) + '\n') # The first title line
    for alphabet_r in sub_alphabet_list:
        r_list = []
        r_list_iden_str = []
        r_list_iden_flo = []
        for alphabet_c in sub_alphabet_list:
            combined = alphabet_r + '_' + alphabet_c
            r_list.append(combined)
            if combined in group_iden_dic:
                r_list_iden_str.append(group_iden_dic[combined])
                r_list_iden_flo.append(float(group_iden_dic[combined]))
            else:
                r_list_iden_str.append('0')
                r_list_iden_flo.append(0)
        members.append(r_list)
        members_iden_str.append(r_list_iden_str)
        members_iden_flo.append(r_list_iden_flo)
        #out.write (alphabet_r + ',' + '%s' % ','.join(r_list_iden_str) + '\n')
        #out.write ('%s' % ','.join(r_list_iden_str) + '\n')
    return members_iden_flo


def get_heatmap_plot(iden_list, group_number):
    data = np.array(iden_list)
    column_labels = list(string.ascii_uppercase)[0: group_number]
    row_labels = list(string.ascii_uppercase)[0: group_number]
    fig, ax = plt.subplots()
    heatmap = ax.pcolor(data)
    plt.colorbar(heatmap)
    ax.set_xticks(np.arange(data.shape[0])+0.5, minor=False) # put the major ticks at the middle of each cell
    ax.set_yticks(np.arange(data.shape[1])+0.5, minor=False) # put the major ticks at the middle of each cell
    ax.invert_yaxis() #setup the direction of Y-axis
    ax.xaxis.tick_top() #put X-axis on the top of the figure
    ax.set_xticklabels(row_labels, minor=False)
    ax.set_yticklabels(column_labels, minor=False)
    plt.show()


group_identity_dic = get_group_identity_dic(group_idens)
list_a = get_identity_list(group_identity_dic,group_numbers)
get_heatmap_plot(list_a,group_numbers)



import os
import argparse
from Bio import SeqIO


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


parser = argparse.ArgumentParser()
parser.add_argument('-in', required=True, help='sequence file')
args = vars(parser.parse_args())
file_in     = args['in']


file_in_path, file_in_basename, file_in_extension = sep_path_basename_ext(file_in)
file_out = '%s/%s_stats.txt' % (file_in_path, file_in_basename)

total_number = 0
total_length = 0
for seq_record in SeqIO.parse(file_in, 'fastq'):
    total_length += len(seq_record.seq)
    total_number += 1


file_out_handle = open(file_out, 'w')
file_out_handle.write('%s\t%s\t%sbp\n' % (file_in, total_number, total_length))
file_out_handle.close()



#!/usr/bin/env python3

import argparse
from BioSAK.global_functions import sep_path_basename_ext


BLCA_op_parser_usage = '''
========== BLCA_op_parser example commands ==========

BioSAK BLCA_op_parser -in OTUs.fasta.blca.out

=====================================================
'''


def BLCA_op_parser(args):

    blca_output = args['in']

    file_in_path, file_in_basename, file_in_ext = sep_path_basename_ext(blca_output)
    output_file_1 = '%s/%s_reformatted_1.txt' % (file_in_path, file_in_basename)
    output_file_2 = '%s/%s_reformatted_2.txt' % (file_in_path, file_in_basename)

    # read in input file
    s16_taxon_blca_dict = {}
    for each_16s_taxon in open(blca_output):
        each_16s_taxon_split = each_16s_taxon.strip().split('\t')
        s16_taxon_blca_dict[each_16s_taxon_split[0]] = each_16s_taxon_split[1]

    taxon_dict_formatted_with_num = {}
    taxon_dict_formatted_no_num = {}
    for each_16s in s16_taxon_blca_dict:
        taxon_blca_raw = s16_taxon_blca_dict[each_16s]
        formatted_taxon_str_with_num = 'Unclassified'
        formatted_taxon_str_no_num = 'Unclassified'
        if taxon_blca_raw != 'Unclassified':
            taxon_blca_raw_split_1 = taxon_blca_raw.strip().split(':')[1:]
            formatted_taxon_list_with_num = []
            formatted_taxon_list_no_num = []
            for each_str in taxon_blca_raw_split_1:
                each_str_split = each_str.split(';')

                # determine_current_rank
                current_rank = ''
                if each_str_split[-1] == 'phylum':
                    current_rank = 'd'
                elif each_str_split[-1] == 'class':
                    current_rank = 'p'
                elif each_str_split[-1] == 'order':
                    current_rank = 'c'
                elif each_str_split[-1] == 'family':
                    current_rank = 'o'
                elif each_str_split[-1] == 'genus':
                    current_rank = 'f'
                elif each_str_split[-1] == 'species':
                    current_rank = 'g'
                elif each_str_split[-1] == '':
                    current_rank = 's'

                taxon_with_confidence = '%s(%s)' % (each_str_split[0], each_str_split[1][:5])
                taxon_without_confidence = '%s__%s' % (current_rank, each_str_split[0])

                formatted_taxon_list_with_num.append(taxon_with_confidence)
                formatted_taxon_list_no_num.append(taxon_without_confidence)

            formatted_taxon_str_with_num = ';'.join(formatted_taxon_list_with_num)
            formatted_taxon_str_no_num = ';'.join(formatted_taxon_list_no_num)

        formatted_taxon_str_with_numno_space = '_'.join(formatted_taxon_str_with_num.split(' '))
        formatted_taxon_str_no_num_no_space = '_'.join(formatted_taxon_str_no_num.split(' '))

        taxon_dict_formatted_with_num[each_16s] = formatted_taxon_str_with_numno_space
        taxon_dict_formatted_no_num[each_16s] = formatted_taxon_str_no_num_no_space

    output_file_1_handle = open(output_file_1, 'w')
    output_file_2_handle = open(output_file_2, 'w')
    for each_seq in taxon_dict_formatted_with_num:
        output_file_1_handle.write('%s\t%s\n' % (each_seq, taxon_dict_formatted_with_num[each_seq]))
        output_file_2_handle.write('%s\t%s\n' % (each_seq, taxon_dict_formatted_no_num[each_seq]))
    output_file_1_handle.close()
    output_file_2_handle.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-in', required=True, help='BLCA output')
    args = vars(parser.parse_args())
    BLCA_op_parser(args)




newick_file = '/Users/songweizhi/Desktop/human_gut_species_tree.newick'
species_list = ['AKV', 'BGC', 'BAD']

subtree_file = '/Users/songweizhi/Desktop/human_gut_species_tree_subset.newick'


###################################################### Biopython #######################################################

from Bio import Phylo

# tree = Phylo.read(newick_file, 'newick')
# common_ancestor = tree.common_ancestor(species_list)
# Phylo.draw_ascii(common_ancestor)


######################################################### ETE3 #########################################################

from ete3 import Tree


def subset_tree(tree_file_in, leaf_node_list, tree_file_out):
    tree_in = Tree(tree_file_in, format=0)
    tree_in.prune(leaf_node_list, preserve_branch_length=True)
    tree_in.write(format=0, outfile=tree_file_out)


subset_tree(newick_file, species_list, subtree_file)




import os
import argparse
from Bio import SeqIO
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


plot_sam_depth_usage = '''
================================== plot_sam_depth example commands ==================================

BioSAK plot_sam_depth -r ref.fa -d 12D9.bam.depth
BioSAK plot_sam_depth -r ref.fa -d 12D9.bam.depth -i contig_01 -s 100 -e 900 -k 300 -l 500,550,600

# get depth file 
module load samtools/1.15
samtools depth 12D9.bam > 12D9.bam.depth

=====================================================================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def take_kmer_mean(num_list, k_mer):

    k_mer_average_list = []
    n = 0

    while n < len(num_list):

        if n <= len(num_list) - k_mer:

            to_add_value = list(range(1, k_mer + 1))

            pos_list = []
            for each in to_add_value:
                pos_list.append(num_list[n + each - 1])

            # get average
            current_average = sum(pos_list) / k_mer
            k_mer_average_list.append(float("{0:.2f}".format(current_average)))

            n += 1

        elif (len(num_list) - k_mer) < n < len(num_list):
            k_mer_average_list.append(num_list[n])

            n += 1

    return k_mer_average_list


def plot_sam(depth_file, seq_to_plot, start_pos, end_pos, bps_to_marker, plot_filename, k_mer, plot_width, plot_height):

    x = []
    y = []
    bp_num = 1
    current_pos = 0
    for each_base in open(depth_file):
        each_base_split = each_base.strip().split('\t')
        seq_id = each_base_split[0]
        pos = int(each_base_split[1])
        depth = int(each_base_split[2])

        if seq_id == seq_to_plot:

            if pos < start_pos:
                pass

            elif pos == start_pos:
                x.append(pos)
                y.append(depth)
                current_pos = pos

            else:
                start_0 = None
                end_0 = None
                to_add = []

                if (pos == current_pos + 1) and (pos <= end_pos):
                    x.append(pos)
                    y.append(depth)
                    current_pos = pos

                elif (pos > current_pos + 1) and (pos <= end_pos):

                    # add zero
                    start_0 = current_pos + 1
                    end_0 = pos - 1
                    to_add = list(range(start_0, end_0 + 1))
                    for each_0 in to_add:
                        x.append(each_0)
                        y.append(0)

                    x.append(pos)
                    y.append(depth)
                    current_pos = pos

                elif (pos > current_pos + 1) and (pos > end_pos):

                    # add zero
                    start_0 = current_pos + 1
                    end_0 = end_pos
                    to_add = list(range(start_0, end_0 + 1))
                    for each_0 in to_add:
                        x.append(each_0)
                        y.append(0)

                    current_pos = pos

    #print('Calculating k-mer means')
    y = take_kmer_mean(y, k_mer)

    cm = 1/2.54
    fig = plt.figure(figsize=(plot_width*cm, plot_height*cm), dpi=100)

    # Change the color and its transparency
    plt.plot(x, y, color="skyblue", alpha=0.7, linewidth=0.7)

    # titles
    plt.title(plot_filename,fontsize=7)
    plt.xlabel('Position (bp)', fontsize=10)
    plt.ylabel('Depth (X)', fontsize=10)

    plt.xticks(fontsize=7)
    plt.yticks(fontsize=7)

    # set axis range
    plt.ylim(0, max(y))

    # add lines to specified positions
    if bps_to_marker != None:
        bps_to_marker_list = bps_to_marker.strip().split(',')
        for each_line in bps_to_marker_list:
            plt.axvline(x=int(each_line), c='red', linewidth=0.2, linestyle='dashed')

    # Get plot
    plt.tight_layout()
    plt.savefig('%s.pdf' % plot_filename, dpi=300)
    plt.close()


def plot_sam_depth(args):

    sequence_file = args['r']
    depth_file = args['d']
    seq_to_plot = args['i']
    start_pos = args['s']
    end_pos = args['e']
    k_mer = args['k']
    bps_to_marker = args['l']
    plot_width = args['x']
    plot_height = args['y']

    # get sequence length dict
    seq_id_length_dict = {}
    for each_seq in SeqIO.parse(sequence_file, 'fasta'):
        seq_id_length_dict[each_seq.id] = len(each_seq.seq)

    if (seq_to_plot != None) and (seq_to_plot not in seq_id_length_dict):
        print('Reference sequence %s not found in reference file, program exited!' % seq_to_plot)
        exit()

    # get depth_file base name
    depth_file_path, depth_file_basename, depth_file_extension = sep_path_basename_ext(depth_file)

    if seq_to_plot != None:

        # get the start and end of the region to plot
        if start_pos == None:
            start_pos = 1
        if end_pos == None:
            end_pos = seq_id_length_dict[seq_to_plot]

        print('Processing %s' % seq_to_plot)
        plot_filename = '%s__%s__%s-%sbp__%smer' % (depth_file_basename, seq_to_plot, start_pos, end_pos, k_mer)
        plot_sam(depth_file, seq_to_plot, start_pos, end_pos, bps_to_marker, plot_filename, k_mer, plot_width, plot_height)

    if seq_to_plot == None:
        for each_ctg in SeqIO.parse(sequence_file, 'fasta'):
            print('Processing %s' % each_ctg.id)
            plot_filename = '%s__%s__%s-%sbp__%smer' % (depth_file_basename, each_ctg.id, 1, seq_id_length_dict[each_ctg.id], k_mer)
            plot_sam(depth_file, each_ctg.id, 1, seq_id_length_dict[each_ctg.id], bps_to_marker, plot_filename, k_mer, plot_width, plot_height)


if __name__ == '__main__':

    plot_sam_depth_parser = argparse.ArgumentParser()

    plot_sam_depth_parser.add_argument('-r',            required=True, type=str,                        help='reference sequence file')
    plot_sam_depth_parser.add_argument('-d',            required=True, type=str,                        help='depth file')
    plot_sam_depth_parser.add_argument('-i',            required=False, type=str, default=None,         help='id of sequence to plot')
    plot_sam_depth_parser.add_argument('-s',            required=False, type=int, default=None,         help='start position to plot')
    plot_sam_depth_parser.add_argument('-e',            required=False, type=int, default=None,         help='end position to plot')
    plot_sam_depth_parser.add_argument('-k',            required=False, type=int, default=100,          help='k-mer mean depth, default: 100')
    plot_sam_depth_parser.add_argument('-l',            required=False, type=str, default=None,         help='position to mark')
    plot_sam_depth_parser.add_argument('-x',            required=False, type=float, default=30,         help='plot width, default: 30')
    plot_sam_depth_parser.add_argument('-y',            required=False, type=float, default=10,         help='plot height, default: 10')
    args = vars(plot_sam_depth_parser.parse_args())
    plot_sam_depth(args)




import seaborn as sns

sns.palplot(sns.color_palette("PuBu", 10))


import seaborn as sns
#print(sns.color_palette("Reds", 15))

sns.color_palette("bone", 15)
sns.color_palette("Reds", 15)
sns.color_palette("flare", 15)



sns.color_palette("light:#5A9", 15)


sns.color_palette("Greens", 15)
sns.dark_palette("seagreen")



# Greys, Purples, Blues, Greens, Oranges, Reds
sns.color_palette("Greys", 5)
sns.color_palette("Purples", 5)
sns.color_palette("Blues", 5)
sns.color_palette("Greens", 5)
sns.color_palette("Reds", 5)


sns.color_palette("Purples", 5)


sns.light_palette("red")



sns.color_palette("Cyans", 15)

